@@ -26,6 +26,7 @@
 #include "tla/libarch/chatter.h"
 #include "tla/libarch/namespace.h"
 #include "tla/libarch/archives.h"
+#include "tla/libarch/archive-version.h"
 #include "tla/libarch/pfs.h"
 #include "tla/libarch/archive-pfs.h"
 #include "tla/libarch/pfs-signatures.h"
@@ -258,7 +259,9 @@ pfs_archive_version (struct arch_archive * a)
   else if (arch_pfs_file_exists (arch->pfs, meta_info_path))
     {
       /* most likely a mirror that's missing a .archive-version file */
-      contents = str_save (0, "Hackerlab arch archive directory, format version 1.");
+      safe_printfmt (2, "Failed to access file '%s'\n", path);
+      safe_printfmt (2, "Could not determine archive format, guessing to be tla 1.0\n");
+      contents = str_save (0, arch_tree_format_1_str);
     }
   else
     {
@@ -282,6 +285,33 @@ pfs_archive_version (struct arch_archive * a)
   return contents;
 }
 
+static rel_table
+pfs_baz_search (struct arch_archive * a,
+                const t_uchar * prefix,
+                enum arch_valid_package_name_types req_type,
+                enum arch_parse_package_name_type ret_type)
+{
+  struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
+  rel_table files = rel_table_nil;
+  rel_table answer = rel_table_nil;
+  int x;
+
+  files = arch_pfs_directory_files (arch->pfs, ".", 0);
+
+  for (x = 0; x < rel_n_records (files); ++x)
+    {
+      const t_uchar * file;
+
+      file = rel_peek_str (files, x, 0);
+      if (   arch_valid_package_name (file, arch_no_archive, req_type, 1)
+          && !str_cmp_prefix (prefix, file))
+        rel_add_records (&answer, rel_make_record_1_taking (rel_make_field_str (arch_parse_package_name (ret_type, NULL, file))), 0);
+    }
+
+  rel_free_table (files);
+  rel_uniq_by_field (&answer, 0);
+  return answer;
+}
 
 static rel_table
 pfs_categories (struct arch_archive * a)
@@ -290,6 +320,9 @@ pfs_categories (struct arch_archive * a)
   rel_table files  = rel_table_nil;
   rel_table answer  = rel_table_nil;
 
+  if (a->type == arch_archive_baz)
+    return pfs_baz_search (a, 0, arch_req_category, arch_ret_category);
+
   files = arch_pfs_directory_files (arch->pfs, ".", 0);
   answer = arch_pick_categories_by_field (files, 0);
 
@@ -306,13 +339,26 @@ pfs_branches (struct arch_archive * a,
   t_uchar * category_path = 0;
   rel_table files  = rel_table_nil;
   rel_table answer  = rel_table_nil;
+  t_uchar * search_prefix = 0;
 
-  category_path = arch_fs_archive_category_path (0, category);
+  if (a->type == arch_archive_baz)
+    {
+      search_prefix = str_alloc_cat_many (0, category, "--", str_end);
 
-  files = arch_pfs_directory_files (arch->pfs, category_path, 0);
-  answer = arch_pick_branches_by_field (files, 0);
+      answer = pfs_baz_search (a, search_prefix, arch_req_package, arch_ret_package);
+    }
+  else
+    {
+      category_path = arch_fs_archive_category_path (a, 0, category);
 
+      files = arch_pfs_directory_files (arch->pfs, category_path, 1);
+      answer = arch_pick_branches_by_field (files, 0);
+
+    }
+
+  lim_free (0, category_path);
   rel_free_table (files);
+  lim_free (0, search_prefix);
   return answer;
 }
 
@@ -325,13 +371,25 @@ pfs_versions (struct arch_archive * a,
   t_uchar * branch_path = 0;
   rel_table files  = rel_table_nil;
   rel_table answer  = rel_table_nil;
+  t_uchar * search_prefix = 0;
 
-  branch_path = arch_fs_archive_branch_path (0, package);
+  if (a->type == arch_archive_baz)
+    {
+      search_prefix = str_alloc_cat_many (0, package, "--", str_end);
 
-  files = arch_pfs_directory_files (arch->pfs, branch_path, 0);
-  answer = arch_pick_versions_by_field (files, 0);
+      answer = pfs_baz_search (a, search_prefix, arch_req_version, arch_ret_package_version);
+    }
+  else
+    {
+      branch_path = arch_fs_archive_branch_path (a, 0, package);
 
+      files = arch_pfs_directory_files (arch->pfs, branch_path, 1);
+      answer = arch_pick_versions_by_field (files, 0);
+    }
+
+  lim_free (0, branch_path);
   rel_free_table (files);
+  lim_free (0, search_prefix);
   return answer;
 }
 
@@ -345,11 +403,12 @@ pfs_revisions (struct arch_archive * a,
   rel_table files  = rel_table_nil;
   rel_table answer  = rel_table_nil;
 
-  version_path = arch_fs_archive_version_path (0, version);
+  version_path = arch_fs_archive_version_path (a, 0, version);
 
   files = arch_pfs_directory_files (arch->pfs, version_path, 0);
   answer = arch_pick_patch_levels_by_field (files, 0);
 
+  lim_free (0, version_path);
   rel_free_table (files);
   return answer;
 }
@@ -369,7 +428,7 @@ pfs_archive_log (struct arch_archive * a,
       exit (2);
     }
 
-  revision_log_path = arch_fs_archive_revision_log_path (0, revision);
+  revision_log_path = arch_fs_archive_revision_log_path (a, 0, revision);
 
   answer = arch_pfs_checked_file_contents (arch, revision, revision_log_path);
 
@@ -404,11 +463,11 @@ pfs_revision_type (enum arch_revision_type * type,
   int has_log_file;
   int result = 0;
 
-  continuation_path = arch_fs_archive_continuation_path (0, revision);
-  changeset_path = arch_fs_archive_changeset_path (0, revision);
-  import_path = arch_fs_archive_import_path (0, revision);
-  cacherev_path = arch_fs_archive_cached_path (0, revision);
-  log_path = arch_fs_archive_revision_log_path (0, revision);
+  continuation_path = arch_fs_archive_continuation_path (a, 0, revision);
+  changeset_path = arch_fs_archive_changeset_path (a, 0, revision);
+  import_path = arch_fs_archive_import_path (a, 0, revision);
+  cacherev_path = arch_fs_archive_cached_path (a, 0, revision);
+  log_path = arch_fs_archive_revision_log_path (a, 0, revision);
 
   continuation_tail = file_name_tail (0, continuation_path);
   changeset_tail = file_name_tail (0, changeset_path);
@@ -537,7 +596,7 @@ pfs_get_patch (int out_fd,
       exit (2);
     }
 
-  changeset_path = arch_fs_archive_changeset_path (0, revision);
+  changeset_path = arch_fs_archive_changeset_path (a, 0, revision);
   arch_pfs_checked_get_file (arch, revision, out_fd, changeset_path);
 
   lim_free (0, changeset_path);
@@ -558,7 +617,7 @@ pfs_get_cached (int out_fd,
       exit (2);
     }
 
-  cached_path = arch_fs_archive_cached_path (0, revision);
+  cached_path = arch_fs_archive_cached_path (a, 0, revision);
   arch_pfs_checked_get_file (arch, revision, out_fd, cached_path);
 
   lim_free (0, cached_path);
@@ -579,7 +638,7 @@ pfs_get_import (int out_fd,
       exit (2);
     }
 
-  import_path = arch_fs_archive_import_path (0, revision);
+  import_path = arch_fs_archive_import_path (a, 0, revision);
   arch_pfs_checked_get_file (arch, revision, out_fd, import_path);
 
   lim_free (0, import_path);
@@ -600,7 +659,7 @@ pfs_get_continuation (struct arch_archive * a,
       exit (2);
     }
 
-  continuation_path = arch_fs_archive_continuation_path (0, revision);
+  continuation_path = arch_fs_archive_continuation_path (a, 0, revision);
 
   answer = arch_pfs_checked_file_contents (arch, revision, continuation_path);
 
@@ -644,7 +703,7 @@ pfs_make_category (t_uchar ** errstr,
 
   invariant (arch_valid_package_name (category, arch_no_archive, arch_req_category, 0));
 
-  cat_path = arch_fs_archive_category_path (0, category);
+  cat_path = arch_fs_archive_category_path (a, 0, category);
 
   if (arch_pfs_file_exists (arch->pfs, cat_path))
     {
@@ -711,7 +770,7 @@ pfs_make_branch (t_uchar ** errstr,
 
   invariant (arch_valid_package_name (branch, arch_no_archive, arch_req_package, 0));
 
-  branch_path = arch_fs_archive_branch_path (0, branch);
+  branch_path = arch_fs_archive_branch_path (a, 0, branch);
 
   if (arch_pfs_file_exists (arch->pfs, branch_path))
     {
@@ -768,7 +827,7 @@ pfs_make_version (t_uchar ** errstr,
 
   invariant (arch_valid_package_name (version, arch_no_archive, arch_req_version, 0));
 
-  version_path = arch_fs_archive_version_path (0, version);
+  version_path = arch_fs_archive_version_path (a, 0, version);
 
   if (arch_pfs_file_exists (arch->pfs, version_path))
     {
@@ -805,7 +864,10 @@ pfs_make_version (t_uchar ** errstr,
       if (arch->arch.http_blows)
         {
           arch_pfs_update_listing_file (arch->pfs, version_path);
-          arch_pfs_update_listing_file (arch->pfs, version_path_dir);
+          if (version_path_dir)
+            arch_pfs_update_listing_file (arch->pfs, version_path_dir);
+          else
+            arch_pfs_update_listing_file (arch->pfs, ".");
         }
 
       lim_free (0, version_tmp_path);
@@ -965,11 +1027,11 @@ pfs_lock_revision (t_uchar ** errstr, struct arch_archive * a,
    *
    */
 
-  unlocked_path = arch_fs_archive_revision_lock_unlocked_path (0, version, prev_level);
-  persistent_locked_path = arch_fs_archive_revision_lock_locked_path (0, version, prev_level, uid, 0);
-  final_locked_path = arch_fs_archive_revision_lock_locked_path (0, version, prev_level, uid, txn_id);
-  final_locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
-  broken_path = arch_fs_archive_revision_lock_broken_path (0, version, prev_level);
+  unlocked_path = arch_fs_archive_revision_lock_unlocked_path (a, 0, version, prev_level);
+  persistent_locked_path = arch_fs_archive_revision_lock_locked_path (a, 0, version, prev_level, uid, 0);
+  final_locked_path = arch_fs_archive_revision_lock_locked_path (a, 0, version, prev_level, uid, txn_id);
+  final_locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
+  broken_path = arch_fs_archive_revision_lock_broken_path (a, 0, version, prev_level);
 
   /* final_locked_path is [D] or [E] as appropriate to the txn_id argument.
    *
@@ -1089,14 +1151,14 @@ pfs_finish_revision (t_uchar ** errstr, struct arch_archive * a,
    *  [B,C,D] -> error
    */
 
-  locked_path = arch_fs_archive_revision_lock_locked_path (0, version, prev_level, uid, txn_id);
-  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
+  locked_path = arch_fs_archive_revision_lock_locked_path (a, 0, version, prev_level, uid, txn_id);
+  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
 
   new_lock_path = file_name_in_vicinity (0, locked_contents_path, "++revision-lock");
   new_contents_path = file_name_in_vicinity (0, new_lock_path, "+contents");
 
   revision = str_alloc_cat_many (0, version, "--", new_level, str_end);
-  new_rev_path = arch_fs_archive_revision_path (0, revision);
+  new_rev_path = arch_fs_archive_revision_path (a, 0, revision);
 
   if (!arch_pfs_file_exists (arch->pfs, locked_path))
     {
@@ -1129,7 +1191,7 @@ pfs_finish_revision (t_uchar ** errstr, struct arch_archive * a,
                 {
                   t_uchar * prev_rev_path = 0;
 
-                  prev_rev_path = arch_fs_archive_revision_path (0, prev_revision);
+                  prev_rev_path = arch_fs_archive_revision_path (a, 0, prev_revision);
                   arch_pfs_update_listing_file (arch->pfs, prev_rev_path);
 
                   lim_free (0, prev_rev_path);
@@ -1192,9 +1254,9 @@ pfs_lock_state (t_uchar ** prev_level_ret,
   if (prev_level_ret)
     *prev_level_ret = prev_level ? str_save (0, prev_level) : 0;
 
-  unlocked_path = arch_fs_archive_revision_lock_unlocked_path (0, version, prev_level);
+  unlocked_path = arch_fs_archive_revision_lock_unlocked_path (a, 0, version, prev_level);
   unlocked_contents_path = file_name_in_vicinity (0, unlocked_path, "+contents");
-  broken_path = arch_fs_archive_revision_lock_broken_path (0, version, prev_level);
+  broken_path = arch_fs_archive_revision_lock_broken_path (a, 0, version, prev_level);
   broken_contents_path = file_name_in_vicinity (0, broken_path, "+contents");
 
   if (uid_ret)
@@ -1213,13 +1275,13 @@ pfs_lock_state (t_uchar ** prev_level_ret,
       t_uchar * persistent_locked_path_stem = 0;
       int x;
 
-      version_path = arch_fs_archive_version_path (0, version);
+      version_path = arch_fs_archive_version_path (a, 0, version);
       files = arch_pfs_directory_files (arch->pfs, version_path, 1);
 
       {
         t_uchar * t = 0;
 
-        t = arch_fs_archive_revision_lock_locked_path (0, version, prev_level, 0, 0);
+        t = arch_fs_archive_revision_lock_locked_path (a, 0, version, prev_level, 0, 0);
         persistent_locked_path_stem = file_name_tail (0, t);
 
         lim_free (0, t);
@@ -1318,9 +1380,9 @@ pfs_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
    *  [A,B,C] -> error
    */
 
-  locked_path = arch_fs_archive_revision_lock_locked_path (0, version, prev_level, uid, txn_id);
-  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
-  unlocked_path = arch_fs_archive_revision_lock_unlocked_path (0, version, prev_level);
+  locked_path = arch_fs_archive_revision_lock_locked_path (a, 0, version, prev_level, uid, txn_id);
+  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
+  unlocked_path = arch_fs_archive_revision_lock_unlocked_path (a, 0, version, prev_level);
 
   if (1 != arch_pfs_is_dir (arch->pfs, locked_contents_path))
     {
@@ -1342,8 +1404,8 @@ pfs_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
       new_lock_path = file_name_in_vicinity (0, locked_contents_path, remade_basename);
       new_lock_contents_path = file_name_in_vicinity (0, new_lock_path, "+contents");
 
-      version_path = arch_fs_archive_version_path (0, version);
-      broken_lock_path = arch_fs_archive_revision_lock_broken_path (0, version, prev_level);
+      version_path = arch_fs_archive_version_path (a, 0, version);
+      broken_lock_path = arch_fs_archive_revision_lock_broken_path (a, 0, version, prev_level);
       broken_lock_dir = file_name_directory_file (0, broken_lock_path);
       junk_path = tmp_file_name (version_path, ",,junk");
 
@@ -1533,7 +1595,7 @@ pfs_put_log (t_uchar ** errstr, struct arch_archive * a,
 
   invariant (!!txn_id);
 
-  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
+  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
   log_path = file_name_in_vicinity (0, locked_contents_path, "log");
 
   tmp_path = tmp_file_name ("/tmp", ",,pfs-dav-put-log");
@@ -1574,7 +1636,7 @@ pfs_put_continuation (t_uchar ** errstr, struct arch_archive * a,
 
   invariant (!!txn_id);
 
-  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
+  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
   continuation_path = file_name_in_vicinity (0, locked_contents_path, "CONTINUATION");
 
   tmp_path = tmp_file_name ("/tmp", ",,pfs-dav-put-log");
@@ -1614,7 +1676,7 @@ pfs_put_changeset (t_uchar ** errstr, struct arch_archive * a,
 
   invariant (!!txn_id);
 
-  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
+  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
   patches = str_alloc_cat_many (0, version, "--", level, ".patches.tar.gz", str_end);
   changeset_path = file_name_in_vicinity (0, locked_contents_path, patches);
 
@@ -1646,7 +1708,7 @@ pfs_put_import (t_uchar ** errstr, struct arch_archive * a,
 
   invariant (!!txn_id);
 
-  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
+  locked_contents_path = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
   import = str_alloc_cat_many (0, version, "--", level, ".src.tar.gz", str_end);
   import_path = file_name_in_vicinity (0, locked_contents_path, import);
 
@@ -1674,7 +1736,7 @@ pfs_put_cached (t_uchar ** errstr,
   t_uchar * cached_tail = 0;
   t_uchar * cached_dir = 0;
 
-  cached_path = arch_fs_archive_cached_path (0, revision);
+  cached_path = arch_fs_archive_cached_path (a, 0, revision);
   cached_tail = file_name_tail (0, cached_path);
   cached_dir = file_name_directory_file (0, cached_path);
 
@@ -1704,8 +1766,8 @@ pfs_delete_cached (t_uchar ** errstr,
   t_uchar * cached_path = 0;
   t_uchar * cached_checksum_path = 0;
 
-  cached_path = arch_fs_archive_cached_path (0, revision);
-  cached_checksum_path = arch_fs_archive_cached_checksum_path (0, revision);
+  cached_path = arch_fs_archive_cached_path (a, 0, revision);
+  cached_checksum_path = arch_fs_archive_cached_checksum_path (a, 0, revision);
 
   if (arch_pfs_file_exists (arch->pfs, cached_path))
     {
@@ -1729,6 +1791,40 @@ pfs_delete_cached (t_uchar ** errstr,
   return 0;
 }
 
+static void
+pfs_fixup_version (int chatter_fd, struct arch_archive * a, const t_uchar * version)
+{
+  t_uchar * version_path = 0;
+  rel_table revisions = rel_table_nil;
+  int r;
+  struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
+
+  arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for version %s\n", version);
+
+  version_path = arch_fs_archive_version_path (a, 0, version);
+  arch_pfs_update_listing_file (arch->pfs, version_path);
+
+  revisions = pfs_revisions (a, version);
+
+  for (r = 0; r < rel_n_records (revisions); ++r)
+    {
+      t_uchar * package_revision = 0;
+      t_uchar * revision_path = 0;
+
+      package_revision = str_alloc_cat_many (0, version, "--", rel_peek_str (revisions, r, 0), str_end);
+      arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for revisions %s\n", package_revision);
+
+      revision_path = arch_fs_archive_revision_path (a, 0, package_revision);
+      arch_pfs_update_listing_file (arch->pfs, revision_path);
+
+      lim_free (0, package_revision);
+      lim_free (0, revision_path);
+    }
+
+
+  lim_free (0, version_path);
+  rel_free_table (revisions);
+}
 
 static void
 pfs_repair_non_txnal (int chatter_fd,
@@ -1759,8 +1855,11 @@ pfs_repair_non_txnal (int chatter_fd,
 
       arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for category %s\n", rel_peek_str (categories, c, 0));
 
-      cat_path = arch_fs_archive_category_path (0, rel_peek_str (categories, c, 0));
-      arch_pfs_update_listing_file (arch->pfs, cat_path);
+      if (a->type != arch_archive_baz)
+        {
+          cat_path = arch_fs_archive_category_path (a, 0, rel_peek_str (categories, c, 0));
+          arch_pfs_update_listing_file (arch->pfs, cat_path);
+        }
 
       branches = pfs_branches (a, rel_peek_str (categories, c, 0));
 
@@ -1772,42 +1871,17 @@ pfs_repair_non_txnal (int chatter_fd,
 
           arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for branch %s\n", rel_peek_str (branches, b, 0));
 
-          branch_path = arch_fs_archive_branch_path (0, rel_peek_str (branches, b, 0));
-          arch_pfs_update_listing_file (arch->pfs, branch_path);
+          if (a->type != arch_archive_baz)
+            {
+              branch_path = arch_fs_archive_branch_path (a, 0, rel_peek_str (branches, b, 0));
+              arch_pfs_update_listing_file (arch->pfs, branch_path);
+            }
 
           versions = pfs_versions (a, rel_peek_str (branches, b, 0));
 
           for (v = 0; v < rel_n_records (versions); ++v)
             {
-              t_uchar * version_path = 0;
-              rel_table revisions  = rel_table_nil;
-              int r;
-
-              arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for version %s\n", rel_peek_str (versions, v, 0));
-
-              version_path = arch_fs_archive_version_path (0, rel_peek_str (versions, v, 0));
-              arch_pfs_update_listing_file (arch->pfs, version_path);
-
-              revisions = pfs_revisions (a, rel_peek_str (versions, v, 0));
-
-              for (r = 0; r < rel_n_records (revisions); ++r)
-                {
-                  t_uchar * package_revision = 0;
-                  t_uchar * revision_path = 0;
-
-                  package_revision = str_alloc_cat_many (0, rel_peek_str (versions, v, 0), "--", rel_peek_str (revisions, r, 0), str_end);
-                  arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for revisions %s\n", package_revision);
-
-                  revision_path = arch_fs_archive_revision_path (0, package_revision);
-                  arch_pfs_update_listing_file (arch->pfs, revision_path);
-
-                  lim_free (0, package_revision);
-                  lim_free (0, revision_path);
-                }
-
-
-              lim_free (0, version_path);
-              rel_free_table (revisions);
+              pfs_fixup_version (chatter_fd, a, rel_peek_str (versions, v, 0));
             }
 
           lim_free (0, branch_path);
@@ -1909,7 +1983,7 @@ pfs_finish_signature_file (struct arch_archive *a,
 
   revision = str_alloc_cat_many (0, version, "--", new_level, str_end);
 
-  dir = arch_fs_archive_revision_lock_locked_contents_path (0, version, prev_level, uid, txn_id);
+  dir = arch_fs_archive_revision_lock_locked_contents_path (a, 0, version, prev_level, uid, txn_id);
   pfs_finish_signature_file_worker (a, revision, dir, "checksum");
 
   lim_free (0, dir);
