@@ -57,7 +57,8 @@ arch_make_archive (const t_uchar * name,
                    const t_uchar * location,
                    const t_uchar * mirror_of,
                    int dot_listing_lossage,
-                   int signed_archive)
+                   int signed_archive,
+                   int tla_archive)
 {
   t_uchar * current_loc = 0;
   t_uchar * version;
@@ -72,7 +73,7 @@ arch_make_archive (const t_uchar * name,
       exit (2);
     }
 
-  version = arch_archive_version_for_new_archive ();
+  version = arch_archive_version_for_new_archive (tla_archive);
 
   arch_pfs_make_archive (name, location, version, mirror_of, dot_listing_lossage, signed_archive);
 
@@ -101,6 +102,7 @@ arch_archive_connect_location (const t_uchar * name,
 
   answer->version = answer->vtable->archive_version (answer);
   answer->access = arch_archive_access (answer->version);
+  answer->type = arch_archive_type (answer->version);
 
   if (answer->access == arch_archive_incompatible)
     {
@@ -1312,6 +1314,310 @@ arch_get_meta_int_info(struct arch_archive * arch,
   return result;
 }
 
+t_uchar *
+arch_fs_archive_category_path (const struct arch_archive * arch,
+                               const t_uchar * archive_path,
+                               const t_uchar * category)
+{
+  if (arch->type == arch_archive_baz)
+    panic ("baz does not support stand alone categories");
+  return file_name_in_vicinity (0, archive_path, category);
+}
+
+t_uchar *
+arch_fs_archive_branch_path (const struct arch_archive * arch,
+                             const t_uchar * archive_path,
+                             const t_uchar * package)
+{
+  t_uchar * category = 0;
+  t_uchar * category_path = 0;
+  t_uchar * branch_path = 0;
+
+  if (arch->type == arch_archive_baz)
+    panic ("baz does not support stand alone branches");
+
+  invariant (arch_valid_package_name (package, arch_no_archive, arch_req_package, 0));
+
+  category = arch_parse_package_name (arch_ret_category, 0, package);
+  category_path = arch_fs_archive_category_path (arch, archive_path, category);
+  branch_path = file_name_in_vicinity (0, category_path, package);
+
+  lim_free (0, category);
+  lim_free (0, category_path);
+  return branch_path;
+}
+
+
+t_uchar *
+arch_fs_archive_version_path (const struct arch_archive * arch,
+                              const t_uchar * archive_path,
+                              const t_uchar * version)
+{
+  t_uchar * package = 0;
+  t_uchar * package_path = 0;
+  t_uchar * version_path = 0;
+
+  invariant (arch_valid_package_name (version, arch_no_archive, arch_req_version, 0));
+  
+  if (arch->type == arch_archive_baz)
+    return str_save (0, version);
+
+  package = arch_parse_package_name (arch_ret_package, 0, version);
+  package_path = arch_fs_archive_branch_path (arch, archive_path, package);
+  version_path = file_name_in_vicinity (0, package_path, version);
+
+  lim_free (0, package);
+  lim_free (0, package_path);
+
+  return version_path;
+}
+
+t_uchar *
+arch_fs_archive_revision_path (const struct arch_archive * arch,
+                               const t_uchar * archive_path,
+                               const t_uchar * revision)
+{
+  t_uchar * version = 0;
+  t_uchar * version_path = 0;
+  t_uchar * level = 0;
+  t_uchar * revision_path = 0;
+
+  invariant (arch_valid_package_name (revision, arch_no_archive, arch_req_patch_level, 0));
+
+  version = arch_parse_package_name (arch_ret_package_version, 0, revision);
+  version_path = arch_fs_archive_version_path (arch, archive_path, version);
+  level = arch_parse_package_name (arch_ret_patch_level, 0, revision);
+  revision_path = file_name_in_vicinity (0, version_path, level);
+
+  lim_free (0, version);
+  lim_free (0, version_path);
+  lim_free (0, level);
+
+  return revision_path;
+}
+
+t_uchar *
+arch_fs_archive_revision_log_path (const struct arch_archive * arch,
+                                   const t_uchar * archive_path,
+                                   const t_uchar * revision)
+{
+  t_uchar * revision_path = 0;
+  t_uchar * log_path = 0;
+
+  revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+  log_path = file_name_in_vicinity (0, revision_path, "log");
+
+  lim_free (0, revision_path);
+
+  return log_path;
+}
+
+t_uchar *
+arch_fs_archive_changeset_path (const struct arch_archive * arch,
+                                const t_uchar * archive_path,
+                                const t_uchar * revision)
+{
+  t_uchar * revision_path = 0;
+  t_uchar * changeset_path;
+
+  revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+  changeset_path = file_name_in_vicinity (0, revision_path, revision);
+  changeset_path = str_realloc_cat (0, changeset_path, ".patches.tar.gz");
+
+  lim_free (0, revision_path);
+
+  return changeset_path;
+}
+
+t_uchar *
+arch_fs_archive_import_path (const struct arch_archive * arch,
+                             const t_uchar * archive_path,
+                             const t_uchar * revision)
+{
+  t_uchar * revision_path = 0;
+  t_uchar * changeset_path;
+
+  revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+  changeset_path = file_name_in_vicinity (0, revision_path, revision);
+  changeset_path = str_realloc_cat (0, changeset_path, ".src.tar.gz");
+
+  lim_free (0, revision_path);
+
+  return changeset_path;
+}
+
+
+t_uchar *
+arch_fs_archive_cached_path (const struct arch_archive * arch,
+                             const t_uchar * archive_path,
+                             const t_uchar * revision)
+{
+  t_uchar * revision_path = 0;
+  t_uchar * changeset_path;
+
+  revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+  changeset_path = file_name_in_vicinity (0, revision_path, revision);
+  changeset_path = str_realloc_cat (0, changeset_path, ".tar.gz");
+
+  lim_free (0, revision_path);
+
+  return changeset_path;
+}
+
+t_uchar *
+arch_fs_archive_cached_checksum_path (const struct arch_archive * arch,
+                                      const t_uchar * archive_path,
+                                      const t_uchar * revision)
+{
+  t_uchar * revision_path = 0;
+  t_uchar * checksum_path = 0;
+
+  revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+  checksum_path = file_name_in_vicinity (0, revision_path, "checksum.cacherev");
+
+  lim_free (0, revision_path);
+
+  return checksum_path;
+}
+
+
+t_uchar *
+arch_fs_archive_continuation_path (const struct arch_archive * arch,
+                                   const t_uchar * archive_path,
+                                   const t_uchar * revision)
+{
+  t_uchar * revision_path = 0;
+  t_uchar * continuation_path;
+
+  revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+  continuation_path = file_name_in_vicinity (0, revision_path, "CONTINUATION");
+
+  lim_free (0, revision_path);
+
+  return continuation_path;
+}
+
+
+
+t_uchar *
+arch_fs_archive_revision_lock_unlocked_path (const struct arch_archive * arch,
+					     const t_uchar * archive_path,
+                                             const t_uchar * version,
+                                             const t_uchar * prev_level)
+{
+  t_uchar * answer = 0;
+
+  invariant (arch_valid_package_name (version, arch_no_archive, arch_req_version, 0));
+
+  if (!prev_level)
+    {
+      t_uchar * version_path = 0;
+
+      version_path = arch_fs_archive_version_path (arch, archive_path, version);
+      answer = file_name_in_vicinity (0, version_path, "++revision-lock");
+
+      lim_free (0, version_path);
+    }
+  else
+    {
+      t_uchar * revision = 0;
+      t_uchar * revision_path = 0;
+
+      revision = str_alloc_cat_many (0, version, "--", prev_level, str_end);
+      revision_path = arch_fs_archive_revision_path (arch, archive_path, revision);
+
+      answer = file_name_in_vicinity (0, revision_path, "++revision-lock");
+
+      lim_free (0, revision);
+      lim_free (0, revision_path);
+    }
+
+  return answer;
+}
+
+
+t_uchar *
+arch_fs_archive_revision_lock_locked_path (const struct arch_archive * arch,
+					   const t_uchar * archive_path,
+                                           const t_uchar * version,
+                                           const t_uchar * prev_level,
+                                           const t_uchar * arch_user_id,
+                                           const t_uchar * txn_id)
+{
+  t_uchar * version_path = 0;
+  t_uchar * lock_basename = 0;
+  t_uchar * answer = 0;
+
+  invariant (arch_valid_package_name (version, arch_no_archive, arch_req_version, 0));
+
+  version_path = arch_fs_archive_version_path (arch, archive_path, version);
+
+  lock_basename = str_alloc_cat_many (0,
+                                      "++revision-lock-held--",
+                                      (prev_level ? prev_level : (t_uchar *)"absolute-0"),
+                                      "--",
+                                      arch_user_id,
+                                      (txn_id ? "--" : 0),
+                                      txn_id,
+                                      str_end);
+
+  answer = file_name_in_vicinity (0, version_path, lock_basename);
+
+  lim_free (0, version_path);
+  lim_free (0, lock_basename);
+  return answer;
+}
+
+
+t_uchar *
+arch_fs_archive_revision_lock_locked_contents_path (const struct arch_archive * arch,
+						    const t_uchar * archive_path,
+                                                    const t_uchar * version,
+                                                    const t_uchar * prev_level,
+                                                    const t_uchar * arch_user_id,
+                                                    const t_uchar * txn_id)
+{
+  t_uchar * locked_path = 0;
+  t_uchar * answer = 0;
+
+  locked_path = arch_fs_archive_revision_lock_locked_path (arch, archive_path, version, prev_level, arch_user_id, txn_id);
+  answer = file_name_in_vicinity (0, locked_path, "+contents");
+
+  lim_free (0, locked_path);
+  return answer;
+}
+
+
+t_uchar *
+arch_fs_archive_revision_lock_broken_path (const struct arch_archive * arch,
+					   const t_uchar * archive_path,
+                                           const t_uchar * version,
+                                           const t_uchar * prev_level)
+{
+  t_uchar * version_path = 0;
+  t_uchar * lock_dir_basename = 0;
+  t_uchar * lock_basename = 0;
+  t_uchar * broken_dir = 0;
+  t_uchar * answer = 0;
+
+  version_path = arch_fs_archive_version_path (arch, archive_path, version);
+
+  lock_dir_basename = str_alloc_cat (0, "++revision-lock-broken--", (prev_level ? prev_level : (t_uchar *)"absolute-0"));
+  lock_basename = str_alloc_cat (0, ",,remade-lock--", (prev_level ? prev_level : (t_uchar *)"absolute-0"));
+
+  broken_dir = file_name_in_vicinity (0, version_path, lock_dir_basename);
+  answer = file_name_in_vicinity (0, broken_dir, lock_basename);
+
+  lim_free (0, version_path);
+  lim_free (0, lock_dir_basename);
+  lim_free (0, lock_basename);
+  lim_free (0, broken_dir);
+  return answer;
+}
+
+
+
+
 
 
 /* tag: Tom Lord Tue May 20 00:52:06 2003 (archive.c)
