@@ -50,25 +50,25 @@ static void find_renames (rel_table * out, rel_table common, assoc_table orig_di
 int
 arch_any_changes (struct arch_changeset_report * report)
 {
-  return (report->removed_dirs
-          || report->added_dirs
+  return (rel_n_records (report->removed_dirs)
+          || rel_n_records (report->added_dirs)
 
-          || report->removed_files
-          || report->added_files
+          || rel_n_records (report->removed_files)
+          || rel_n_records (report->added_files)
 
-          || report->removed_symlinks
-          || report->added_symlinks
+          || rel_n_records (report->removed_symlinks)
+          || rel_n_records (report->added_symlinks)
 
-          || report->renamed_files
-          || report->renamed_dirs
+          || rel_n_records (report->renamed_files)
+          || rel_n_records (report->renamed_dirs)
 
-          || report->patched_regular_files
-          || report->patched_symlinks
-          || report->patched_binaries
-          || report->file_metadata_changed
-          || report->dir_metadata_changed
-          || report->symlink_to_file
-          || report->file_to_symlink);
+          || rel_n_records (report->patched_regular_files)
+          || rel_n_records (report->patched_symlinks)
+          || rel_n_records (report->patched_binaries)
+          || rel_n_records (report->file_metadata_changed)
+          || rel_n_records (report->dir_metadata_changed)
+          || rel_n_records (report->symlink_to_file)
+          || rel_n_records (report->file_to_symlink));
 }
 
 void
@@ -79,14 +79,14 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
   t_uchar * mod_files_index_path = 0;
   t_uchar * mod_dirs_index_path = 0;
 
-  rel_table removed_dirs_index = 0;
-  rel_table added_dirs_index = 0;
+  rel_table removed_dirs_index = rel_table_nil;
+  rel_table added_dirs_index = rel_table_nil;
 
-  rel_table removed_files_and_symlinks_index = 0;
-  rel_table added_files_and_symlinks_index = 0;
+  rel_table removed_files_and_symlinks_index = rel_table_nil;
+  rel_table added_files_and_symlinks_index = rel_table_nil;
 
-  rel_table common_files_index = 0; /* [0] origloc [1] modloc [2] id */
-  rel_table common_dirs_index = 0; /* [0] origloc [1] modloc [2] id */
+  rel_table common_files_index = rel_table_nil; /* [0] origloc [1] modloc [2] id */
+  rel_table common_dirs_index = rel_table_nil; /* [0] origloc [1] modloc [2] id */
 
   assoc_table orig_dir_loc_of = 0;
   assoc_table orig_dir_id_of = 0;
@@ -132,10 +132,10 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
   {
     t_uchar * original_only_dir_metadata_file = 0;
     t_uchar * modified_only_dir_metadata_file = 0;
-    rel_table original_only_dir_metadata = 0;
-    rel_table modified_only_dir_metadata = 0;
-    rel_table orig_dirs_by_name = 0;
-    rel_table mod_dirs_by_name = 0;
+    rel_table original_only_dir_metadata = rel_table_nil;
+    rel_table modified_only_dir_metadata = rel_table_nil;
+    rel_table orig_dirs_by_name = rel_table_nil;
+    rel_table mod_dirs_by_name = rel_table_nil;
     rel_table t;        /* [0] name [1] id  -- sort 0*/
     rel_table t2;       /* [0] name [1] id [2] name -- sort 0 -- of dirs sans perms */
 
@@ -214,7 +214,7 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
     int x;
     int here_fd;
     t_uchar * patches_root;
-    rel_table patches_files = 0;
+    rel_table patches_files = rel_table_nil;
     assoc_table saw_patches_file = 0;
 
     patches_root = file_name_in_vicinity (0, path, "patches");
@@ -230,7 +230,7 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
     lim = rel_n_records (patches_files);
     for (x = 0; x < lim; ++x)
       {
-        t_uchar * patch_file = 0;
+        const t_uchar * patch_file;
         t_uchar * patch_file_tail = 0;
         t_uchar * patch_file_suffix = 0;
 
@@ -239,10 +239,10 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
         t_uchar * complement_patch = 0;
 
         t_uchar * patch_file_path = 0;
-        t_uchar * id = 0;      /* not allocated */
 
+        rel_field id_field = rel_field_nil;
 
-        patch_file = patches_files[x][0];
+        patch_file = rel_peek_str (patches_files, x, 0);
         patch_file_suffix = str_chr_rindex (patch_file, '.');
         patch_file_tail = file_name_tail (0, patch_file);
 
@@ -253,47 +253,52 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
           {
             patch_file_loc = file_name_directory_file (0, patch_file);
             dest = &report->dir_metadata_changed;
-            id = assoc_ref (mod_dir_id_of, patch_file_loc);
+            id_field = assoc_get_taking (mod_dir_id_of, rel_make_field_str (patch_file_loc));
           }
         else if (!str_cmp (patch_file_suffix, ".link-orig"))
           {
             patch_file_loc = str_save_n (0, patch_file, patch_file_suffix - patch_file);
             complement_patch = str_alloc_cat (0, patch_file_loc, ".link-mod");
-            if (assoc_ref (saw_patches_file, complement_patch))
+            if (assoc_get_str_taking (saw_patches_file, rel_make_field_str (complement_patch)))
               dest = &report->patched_symlinks;
             else
               dest = &report->symlink_to_file;
-            id = assoc_ref (mod_file_id_of, patch_file_loc);
+            id_field = assoc_get_taking (mod_file_id_of, rel_make_field_str (patch_file_loc));
           }
         else if (!str_cmp (patch_file_suffix, ".original"))
           {
             patch_file_loc = str_save_n (0, patch_file, patch_file_suffix - patch_file);
             complement_patch = str_alloc_cat (0, patch_file_loc, ".modified");
-            if (assoc_ref (saw_patches_file, complement_patch))
+            if (assoc_get_str_taking (saw_patches_file, rel_make_field_str (complement_patch)))
               dest = &report->patched_binaries;
             else
               dest = &report->file_to_symlink;
-            id = assoc_ref (mod_file_id_of, patch_file_loc);
+            id_field = assoc_get_taking (mod_file_id_of, rel_make_field_str (patch_file_loc));
           }
         else if (!str_cmp (patch_file_suffix, ".patch"))
           {
             patch_file_loc = str_save_n (0, patch_file, patch_file_suffix - patch_file);
             dest = &report->patched_regular_files;
-            id = assoc_ref (mod_file_id_of, patch_file_loc);
+            id_field = assoc_get_taking (mod_file_id_of, rel_make_field_str (patch_file_loc));
           }
         else if (!str_cmp (patch_file_suffix, ".meta-orig"))
           {
             patch_file_loc = str_save_n (0, patch_file, patch_file_suffix - patch_file);
             dest = &report->file_metadata_changed;
-            id = assoc_ref (mod_file_id_of, patch_file_loc);
+            id_field = assoc_get_taking (mod_file_id_of, rel_make_field_str (patch_file_loc));
           }
 
         if (dest && patch_file_loc)
           {
             patch_file_path = file_name_in_vicinity (0, patches_root, patch_file_loc);
-            rel_add_records (dest, rel_make_record (patch_file_loc, id, patch_file_path, 0), 0);
+            rel_add_records (dest,
+                             rel_make_record_3_taking (rel_make_field_str (patch_file_loc),
+                                                       rel_field_ref (id_field),
+                                                       rel_make_field_str (patch_file_path)),
+                             0);
           }
 
+        rel_field_unref (id_field);
         lim_free (0, patch_file_tail);
         lim_free (0, patch_file_loc);
         lim_free (0, complement_patch);
@@ -380,17 +385,19 @@ arch_reverse_changeset (struct arch_changeset_report * report)
     {
       t_uchar * s;
 
-      s = report->renamed_files[x][0];
-      report->renamed_files[x][0] = report->renamed_files[x][1];
-      report->renamed_files[x][1] = s;
+      s = str_save (0, rel_peek_str (report->renamed_files, x, 0));
+      rel_set_taking (report->renamed_files, x, 0, rel_get_field (report->renamed_files, x, 1));
+      rel_set_taking (report->renamed_files, x, 1, rel_make_field_str (s));
+      lim_free (0, s);
     }
   for (x = 0; x < rel_n_records (report->renamed_dirs); ++x)
     {
       t_uchar * s;
 
-      s = report->renamed_dirs[x][0];
-      report->renamed_dirs[x][0] = report->renamed_dirs[x][1];
-      report->renamed_dirs[x][1] = s;
+      s = str_save (0, rel_peek_str (report->renamed_dirs, x, 0));
+      rel_set_taking (report->renamed_dirs, x, 0, rel_get_field (report->renamed_dirs, x, 1));
+      rel_set_taking (report->renamed_dirs, x, 1, rel_make_field_str (s));
+      lim_free (0, s);
     }
 
   new_mod_file_loc_of = rel_to_assoc (report->mod_files_index, 1, 0);
@@ -412,17 +419,12 @@ fix_mod_locs (assoc_table loc_of, rel_table table)
 
   for (x = 0; x < rel_n_records (table); ++x)
     {
-      t_uchar * new_loc = 0;
-      t_uchar * old_loc = 0;
-      t_uchar * id;
+      rel_field id_field;
+      rel_field new_loc_field;
 
-      id = table[x][1];
-      old_loc = table[x][0];
-      new_loc = assoc_ref (loc_of, id);
-      if (new_loc)
-        new_loc = str_save (0, new_loc);
-      table[x][0] = new_loc;
-      lim_free (0, old_loc);
+      id_field = rel_get_field (table, x, 1);
+      new_loc_field = assoc_get_taking (loc_of, id_field);
+      rel_set_taking (table, x, 0, new_loc_field);
     }
 }
 
@@ -442,11 +444,11 @@ arch_print_file_metadata_diffs (int out_fd,
       t_uchar * file_metadata_changed = 0;
 
       file_metadata_changed = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                                          report->file_metadata_changed[x][0]);
+                                                          rel_peek_str (report->file_metadata_changed, x, 0));
       safe_printfmt (out_fd, "    %s\n", file_metadata_changed);
 
-      orig_meta_path = str_alloc_cat (0, report->file_metadata_changed[x][2], ".meta-orig");
-      mod_meta_path = str_alloc_cat (0, report->file_metadata_changed[x][2], ".meta-mod");
+      orig_meta_path = str_alloc_cat (0, rel_peek_str (report->file_metadata_changed, x, 2), ".meta-orig");
+      mod_meta_path = str_alloc_cat (0, rel_peek_str (report->file_metadata_changed, x, 2), ".meta-mod");
       orig_meta = read_line_from_file (orig_meta_path);
       mod_meta = read_line_from_file (mod_meta_path);
 
@@ -476,11 +478,11 @@ arch_print_dir_metadata_diffs (int out_fd,
       t_uchar * dir_metadata_changed = 0;
 
       dir_metadata_changed = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                                         report->dir_metadata_changed[x][0]);
+                                                         rel_peek_str (report->dir_metadata_changed, x, 0));
       safe_printfmt (out_fd, "    %s\n", dir_metadata_changed);
 
-      orig_meta_path = file_name_in_vicinity (0, report->dir_metadata_changed[x][2], "=dir-meta-orig");
-      mod_meta_path = file_name_in_vicinity (0, report->dir_metadata_changed[x][2], "=dir-meta-mod");
+      orig_meta_path = file_name_in_vicinity (0, rel_peek_str (report->dir_metadata_changed, x, 2), "=dir-meta-orig");
+      mod_meta_path = file_name_in_vicinity (0, rel_peek_str (report->dir_metadata_changed, x, 2), "=dir-meta-mod");
       orig_meta = read_line_from_file (orig_meta_path);
       mod_meta = read_line_from_file (mod_meta_path);
 
@@ -506,7 +508,7 @@ arch_print_file_diffs (int out_fd,
       t_uchar * patch_file;
       int in_fd;
 
-      patch_file = str_alloc_cat (0, report->patched_regular_files[x][2], ".patch");
+      patch_file = str_alloc_cat (0, rel_peek_str (report->patched_regular_files, x, 2), ".patch");
       in_fd = safe_open (patch_file, O_RDONLY, 0);
       copy_fd (in_fd, out_fd);
       safe_printfmt (out_fd, "\n\n");
@@ -524,34 +526,34 @@ arch_print_added_file_diffs (int out_fd,
 
   for (x = 0; x < rel_n_records (report->added_files); ++x)
     {
-      arch_really_invoke_diff (1, "/dev/null", NULL, report->added_files[x][2], NULL);
+      arch_really_invoke_diff (1, "/dev/null", NULL, rel_peek_str (report->added_files, x, 2), NULL);
     }
 }
 
 void
 arch_print_changeset_diffs (int out_fd, struct arch_changeset_report * report, int escape_classes)
 {
-  if (report->file_metadata_changed)
+  if (rel_n_records (report->file_metadata_changed))
     {
       safe_printfmt (out_fd, "* file metadata changed\n\n");
       arch_print_file_metadata_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->dir_metadata_changed)
+  if (rel_n_records (report->dir_metadata_changed))
     {
       safe_printfmt (out_fd, "* dir metadata changed\n\n");
       arch_print_dir_metadata_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->patched_regular_files)
+  if (rel_n_records (report->patched_regular_files))
     {
       safe_printfmt (out_fd, "* modified files\n\n");
       arch_print_file_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
-  if (report->added_files)
+  if (rel_n_records (report->added_files))
     {
       safe_printfmt (out_fd, "* added files\n\n");
       arch_print_added_file_diffs (out_fd, report, escape_classes);
@@ -574,88 +576,90 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
 {
   int x;
 
-  if (report->removed_dirs)
+  if (rel_n_records (report->removed_dirs))
     {
       safe_printfmt (out_fd, "* removed directories\n\n");
       for (x = 0; x < rel_n_records (report->removed_dirs); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->removed_dirs[x][0]);
+                                             rel_peek_str (report->removed_dirs, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
-  if (report->added_dirs)
+  if (rel_n_records (report->added_dirs))
     {
       safe_printfmt (out_fd, "* added directories\n\n");
       for (x = 0; x < rel_n_records (report->added_dirs); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->added_dirs[x][0]);
+                                             rel_peek_str (report->added_dirs, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->removed_files)
+  if (rel_n_records (report->removed_files))
     {
       safe_printfmt (out_fd, "* removed files\n\n");
       for (x = 0; x < rel_n_records (report->removed_files); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->removed_files[x][0]);
+                                             rel_peek_str (report->removed_files, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
-  if (report->added_files)
+
+  if (rel_n_records (report->added_files))
     {
       safe_printfmt (out_fd, "* added files\n\n");
       for (x = 0; x < rel_n_records (report->added_files); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->added_files[x][0]);
+                                             rel_peek_str (report->added_files, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->removed_symlinks)
+  if (rel_n_records (report->removed_symlinks))
     {
       safe_printfmt (out_fd, "* removed symlinks\n\n");
       for (x = 0; x < rel_n_records (report->removed_symlinks); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->removed_symlinks[x][0]);
-          safe_printfmt (out_fd, "    %s\n", no_dot (item));
-          lim_free (0, item);
-        }
-      safe_printfmt (out_fd, "\n");
-    }
-  if (report->added_symlinks)
-    {
-      safe_printfmt (out_fd, "* added symlinks\n\n");
-      for (x = 0; x < rel_n_records (report->added_symlinks); ++x)
-        {
-          t_uchar * item;
-          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->added_symlinks[x][0]);
+                                             rel_peek_str (report->removed_symlinks, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->renamed_files)
+  if (rel_n_records (report->added_symlinks))
+    {
+      safe_printfmt (out_fd, "* added symlinks\n\n");
+      for (x = 0; x < rel_n_records (report->added_symlinks); ++x)
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             rel_peek_str (report->added_symlinks, x, 0));
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
+      safe_printfmt (out_fd, "\n");
+    }
+
+  if (rel_n_records (report->renamed_files))
     {
       safe_printfmt (out_fd, "* renamed files and symlinks\n\n");
       for (x = 0; x < rel_n_records (report->renamed_files); ++x)
@@ -663,15 +667,16 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
           t_uchar * item1;
           t_uchar * item2;
           item1 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                              report->renamed_files[x][0]);
+                                              rel_peek_str (report->renamed_files, x, 0));
           item2 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                              report->renamed_files[x][1]);
+                                              rel_peek_str (report->renamed_files, x, 1));
           safe_printfmt (out_fd, "    %s\n     => %s\n\n", no_dot (item1), no_dot (item2));
           lim_free (0, item2);
           lim_free (0, item1);
         }
     }
-  if (report->renamed_dirs)
+
+  if (rel_n_records (report->renamed_dirs))
     {
       safe_printfmt (out_fd, "* renamed directories\n\n");
       for (x = 0; x < rel_n_records (report->renamed_dirs); ++x)
@@ -679,43 +684,44 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
           t_uchar * item1;
           t_uchar * item2;
           item1 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                              report->renamed_dirs[x][0]);
+                                              rel_peek_str (report->renamed_dirs, x, 0));
           item2 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                              report->renamed_dirs[x][1]);
+                                              rel_peek_str (report->renamed_dirs, x, 1));
           safe_printfmt (out_fd, "    %s\n     => %s\n\n", no_dot (item1), no_dot (item2));
           lim_free (0, item2);
           lim_free (0, item1);
         }
     }
 
-  if (report->file_to_symlink)
+  if (rel_n_records (report->file_to_symlink))
     {
       safe_printfmt (out_fd, "* files replaced by symlinks \n\n");
       for (x = 0; x < rel_n_records (report->file_to_symlink); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->file_to_symlink[x][0]);
-          safe_printfmt (out_fd, "    %s\n", no_dot (item));
-          lim_free (0, item);
-        }
-      safe_printfmt (out_fd, "\n");
-    }
-  if (report->symlink_to_file)
-    {
-      safe_printfmt (out_fd, "* symlinks replaced by files\n\n");
-      for (x = 0; x < rel_n_records (report->symlink_to_file); ++x)
-        {
-          t_uchar * item;
-          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->symlink_to_file[x][0]);
+                                             rel_peek_str (report->file_to_symlink, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->file_metadata_changed)
+  if (rel_n_records (report->symlink_to_file))
+    {
+      safe_printfmt (out_fd, "* symlinks replaced by files\n\n");
+      for (x = 0; x < rel_n_records (report->symlink_to_file); ++x)
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             rel_peek_str (report->symlink_to_file, x, 0));
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
+      safe_printfmt (out_fd, "\n");
+    }
+
+  if (rel_n_records (report->file_metadata_changed))
     {
       safe_printfmt (out_fd, "* file metadata changed\n\n");
 
@@ -729,7 +735,7 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
             {
               t_uchar * item;
               item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                                 report->file_metadata_changed[x][0]);
+                                                 rel_peek_str (report->file_metadata_changed, x, 0));
               safe_printfmt (out_fd, "    %s\n", no_dot (item));
               lim_free (0, item);
             }
@@ -737,7 +743,7 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->dir_metadata_changed)
+  if (rel_n_records (report->dir_metadata_changed))
     {
       safe_printfmt (out_fd, "* dir metadata changed\n\n");
 
@@ -751,7 +757,7 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
             {
               t_uchar * item;
               item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                                 report->dir_metadata_changed[x][0]);
+                                                 rel_peek_str (report->dir_metadata_changed, x, 0));
               safe_printfmt (out_fd, "    %s\n", no_dot (item));
               lim_free (0, item);
             }
@@ -759,33 +765,35 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
       safe_printfmt (out_fd, "\n");
     }
 
-  if (report->patched_symlinks)
+  if (rel_n_records (report->patched_symlinks))
     {
       safe_printfmt (out_fd, "* retargeted symlinks\n\n");
       for (x = 0; x < rel_n_records (report->patched_symlinks); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->patched_symlinks[x][0]);
+                                             rel_peek_str (report->patched_symlinks, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
-  if (report->patched_binaries)
+
+  if (rel_n_records (report->patched_binaries))
     {
       safe_printfmt (out_fd, "* modified binary files\n\n");
       for (x = 0; x < rel_n_records (report->patched_binaries); ++x)
         {
           t_uchar * item;
           item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                             report->patched_binaries[x][0]);
+                                             rel_peek_str (report->patched_binaries, x, 0));
           safe_printfmt (out_fd, "    %s\n", no_dot (item));
           lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
-  if (report->patched_regular_files)
+
+  if (rel_n_records (report->patched_regular_files))
     {
       safe_printfmt (out_fd, "* modified files\n\n");
 
@@ -799,7 +807,7 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
             {
               t_uchar * item;
               item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
-                                                 report->patched_regular_files[x][0]);
+                                                 rel_peek_str (report->patched_regular_files, x, 0));
               safe_printfmt (out_fd, "    %s\n", no_dot (item));
               lim_free (0, item);
             }
@@ -811,30 +819,30 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
 void
 arch_free_changeset_report_data (struct arch_changeset_report * r)
 {
-  rel_free_table (r->orig_files_index);                 r->orig_files_index = 0;
-  rel_free_table (r->orig_dirs_index);                  r->orig_dirs_index = 0;
-  rel_free_table (r->mod_files_index);                  r->mod_files_index = 0;
-  rel_free_table (r->mod_dirs_index);                   r->mod_dirs_index = 0;
+  rel_free_table (r->orig_files_index);                 r->orig_files_index = rel_table_nil;
+  rel_free_table (r->orig_dirs_index);                  r->orig_dirs_index = rel_table_nil;
+  rel_free_table (r->mod_files_index);                  r->mod_files_index = rel_table_nil;
+  rel_free_table (r->mod_dirs_index);                   r->mod_dirs_index = rel_table_nil;
 
-  rel_free_table (r->removed_dirs);                     r->removed_dirs = 0;
-  rel_free_table (r->added_dirs);                       r->added_dirs = 0;
+  rel_free_table (r->removed_dirs);                     r->removed_dirs = rel_table_nil;
+  rel_free_table (r->added_dirs);                       r->added_dirs = rel_table_nil;
 
-  rel_free_table (r->removed_files);                    r->removed_files = 0;
-  rel_free_table (r->added_files);                      r->added_files = 0;
+  rel_free_table (r->removed_files);                    r->removed_files = rel_table_nil;
+  rel_free_table (r->added_files);                      r->added_files = rel_table_nil;
 
-  rel_free_table (r->removed_symlinks);                 r->removed_symlinks = 0;
-  rel_free_table (r->added_symlinks);                   r->added_symlinks = 0;
+  rel_free_table (r->removed_symlinks);                 r->removed_symlinks = rel_table_nil;
+  rel_free_table (r->added_symlinks);                   r->added_symlinks = rel_table_nil;
 
-  rel_free_table (r->renamed_files);                    r->renamed_files = 0;
-  rel_free_table (r->renamed_dirs);                     r->renamed_dirs = 0;
+  rel_free_table (r->renamed_files);                    r->renamed_files = rel_table_nil;
+  rel_free_table (r->renamed_dirs);                     r->renamed_dirs = rel_table_nil;
 
-  rel_free_table (r->patched_regular_files);            r->patched_regular_files = 0;
-  rel_free_table (r->patched_symlinks);                 r->patched_symlinks = 0;
-  rel_free_table (r->patched_binaries);                 r->patched_binaries = 0;
-  rel_free_table (r->file_metadata_changed);            r->file_metadata_changed = 0;
-  rel_free_table (r->dir_metadata_changed);             r->dir_metadata_changed = 0;
-  rel_free_table (r->symlink_to_file);                  r->symlink_to_file = 0;
-  rel_free_table (r->file_to_symlink);                  r->file_to_symlink = 0;
+  rel_free_table (r->patched_regular_files);            r->patched_regular_files = rel_table_nil;
+  rel_free_table (r->patched_symlinks);                 r->patched_symlinks = rel_table_nil;
+  rel_free_table (r->patched_binaries);                 r->patched_binaries = rel_table_nil;
+  rel_free_table (r->file_metadata_changed);            r->file_metadata_changed = rel_table_nil;
+  rel_free_table (r->dir_metadata_changed);             r->dir_metadata_changed = rel_table_nil;
+  rel_free_table (r->symlink_to_file);                  r->symlink_to_file = rel_table_nil;
+  rel_free_table (r->file_to_symlink);                  r->file_to_symlink = rel_table_nil;
 }
 
 
@@ -857,17 +865,25 @@ report_unique_files_and_symlinks (rel_table * files, rel_table * symlinks,
       t_uchar * archive_path;
       struct stat stat_buf;
 
-      archive_path = file_name_in_vicinity (0, archive, index[x][0]);
+      archive_path = file_name_in_vicinity (0, archive, rel_peek_str (index, x, 0));
 
       safe_lstat (archive_path, &stat_buf);
 
       if (S_ISLNK (stat_buf.st_mode))
         {
-          rel_add_records (symlinks, rel_make_record (index[x][0], index[x][1], archive_path, 0), 0);
+          rel_add_records (symlinks,
+                           rel_make_record_3_taking (rel_get_field (index, x, 0),
+                                                     rel_get_field (index, x, 1),
+                                                     rel_make_field_str (archive_path)),
+                           0);
         }
       else
         {
-          rel_add_records (files, rel_make_record (index[x][0], index[x][1], archive_path, 0), 0);
+          rel_add_records (files,
+                           rel_make_record_3_taking (rel_get_field (index, x, 0),
+                                                     rel_get_field (index, x, 1),
+                                                     rel_make_field_str (archive_path)),
+                           0);
         }
 
       lim_free (0, archive_path);
@@ -888,19 +904,29 @@ find_renames (rel_table * out, rel_table common, assoc_table orig_dir_id_of, ass
   lim = rel_n_records (common);
   for (x = 0; x < lim; ++x)
     {
-      t_uchar * orig_loc = 0;
-      t_uchar * mod_loc = 0;
-      t_uchar * id = 0;
+      rel_field orig_loc_field;
+      rel_field mod_loc_field;
+      rel_field id_field;
+
+      const t_uchar * orig_loc;
+      const t_uchar * mod_loc;
+      const t_uchar * id;
+
       t_uchar * orig_basename = 0;
       t_uchar * mod_basename = 0;
       t_uchar * orig_dir = 0;
       t_uchar * mod_dir = 0;
-      t_uchar * orig_dir_id = 0;
-      t_uchar * mod_dir_id = 0;
+      const t_uchar * orig_dir_id;
+      const t_uchar * mod_dir_id;
 
-      orig_loc = common[x][0];
-      mod_loc = common[x][1];
-      id = common[x][2];
+      orig_loc_field = rel_get_field (common, x, 0);
+      orig_loc = rel_field_str (orig_loc_field);
+
+      mod_loc_field = rel_get_field (common, x, 1);
+      mod_loc = rel_field_str (mod_loc_field);
+
+      id_field = rel_get_field (common, x, 2);
+      id = rel_field_str (id_field);
 
       orig_basename = file_name_tail (0, orig_loc);
       mod_basename = file_name_tail (0, mod_loc);
@@ -908,8 +934,8 @@ find_renames (rel_table * out, rel_table common, assoc_table orig_dir_id_of, ass
       orig_dir = file_name_directory_file (0, orig_loc);
       mod_dir = file_name_directory_file (0, mod_loc);
 
-      orig_dir_id = assoc_ref (orig_dir_id_of, orig_dir);
-      mod_dir_id = assoc_ref (mod_dir_id_of, mod_dir);
+      orig_dir_id = assoc_get_str_taking (orig_dir_id_of, rel_make_field_str (orig_dir));
+      mod_dir_id = assoc_get_str_taking (mod_dir_id_of, rel_make_field_str (mod_dir));
 
       /* The conditional subexpression here attempts to identify renames
        * even when the the changeset is missing locs and ids for
@@ -922,10 +948,20 @@ find_renames (rel_table * out, rel_table common, assoc_table orig_dir_id_of, ass
               ? str_cmp (orig_dir_id, mod_dir_id)
               : str_cmp (orig_dir, mod_dir)))
         {
-          rel_add_records (out, rel_make_record (orig_loc, mod_loc, id, 0), 0);
+          rel_field_ref (orig_loc_field);
+          rel_field_ref (mod_loc_field);
+          rel_field_ref (id_field);
+          rel_add_records (out, rel_make_record_3_taking (orig_loc_field,
+                                                          mod_loc_field,
+                                                          id_field),
+                           0);
         }
 
 
+      rel_field_unref (orig_loc_field);
+      rel_field_unref (mod_loc_field);
+      rel_field_unref (id_field);
+
       lim_free (0, orig_basename);
       lim_free (0, mod_basename);
       lim_free (0, orig_dir);
