@@ -53,10 +53,12 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "print the date of each patch") \
   OP (opt_kind, "k", "kind", 0, \
       "show each revision kind (import, changeset or id)") \
+  OP (opt_cacherevs, "C", "cacherevs", 0, \
+      "show cached revisions") \
   OP (opt_full, "f", "full", 0, \
       "print full patch level names") \
   OP (opt_desc, 0, "desc", 0, \
-      "implies -s -c -D -k") \
+      "implies -s -c -D -k -C") \
   OP (opt_local_merges, 0, "local-merges", 0, \
       "list merges from the same archive") \
   OP (opt_foreign_merges, 0, "foreign-merges", 0, \
@@ -117,6 +119,7 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
   int reverse = 0;
   int wanted_headers = 0;
   int show_kind = 0;
+  int show_cacherevs = 0;
   enum abrowse_depth depth = revision_depth;
   t_uchar * since = 0;
   int since_limits_opt = 0;
@@ -203,10 +206,17 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
             break;
           }
 
+        case opt_cacherevs:
+          {
+            show_cacherevs = 1;
+            break;
+          }
+
         case opt_desc:
           {
             wanted_headers |= (arch_include_summary | arch_include_creator | arch_include_date);
             show_kind = 1;
+            show_cacherevs = 1;
             break;
           }
 
@@ -286,8 +296,8 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
     assoc_table limit_versions = 0;
     struct arch_archive * arch = 0;
     assoc_table since_limits = 0;
-    rel_table old_since_limits = 0; /* [0] version [1] level */
-    rel_table new_since_limits = 0; /* [0] version [1] level */
+    rel_table old_since_limits = rel_table_nil; /* [0] version [1] level */
+    rel_table new_since_limits = rel_table_nil; /* [0] version [1] level */
 
 
     /********************************
@@ -372,7 +382,7 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
     if (since)
       {
         int in_fd;
-        rel_table since_table = 0;
+        rel_table since_table = rel_table_nil;
         int s;
 
 
@@ -386,7 +396,7 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
             t_uchar * version = 0;
             t_uchar * level = 0;
 
-            rev = str_save (0, since_table[s][0]);
+            rev = str_save (0, rel_peek_str (since_table, s, 0));
 
             if (!arch_valid_package_name (rev, arch_no_archive, arch_req_version, 1))
               {
@@ -399,8 +409,8 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
             if (arch_valid_package_name (rev, arch_no_archive, arch_req_patch_level, 0))
               {
                 level = arch_parse_package_name (arch_ret_patch_level, 0, rev);
-                assoc_set (&since_limits, version, level);
-                rel_add_records (&old_since_limits, rel_make_record (version, level, 0), 0);
+                assoc_set_taking (&since_limits, rel_make_field_str (version), rel_make_field_str (level));
+                rel_add_records (&old_since_limits, rel_make_record_2_taking (rel_make_field_str (version), rel_make_field_str (level)), 0);
               }
 
             if (since_limits_opt)
@@ -411,9 +421,9 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
                 category = arch_parse_package_name (arch_ret_category, 0, rev);
                 branch = arch_parse_package_name (arch_ret_package, 0, rev);
 
-                assoc_set (&limit_categories, category, "yes");
-                assoc_set (&limit_branches, branch, "yes");
-                assoc_set (&limit_versions, version, "yes");
+                assoc_set_taking (&limit_categories, rel_make_field_str (category), rel_make_field_str ("yes"));
+                assoc_set_taking (&limit_branches, rel_make_field_str (branch), rel_make_field_str ("yes"));
+                assoc_set_taking (&limit_versions, rel_make_field_str (version), rel_make_field_str ("yes"));
 
                 lim_free (0, category);
                 lim_free (0, branch);
@@ -445,7 +455,7 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
     safe_flush (1);
 
     {
-      rel_table categories = 0;
+      rel_table categories = rel_table_nil;
       int c;
       t_uchar * last_category_printed = 0;
       t_uchar * last_branch_printed = 0;
@@ -455,104 +465,110 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
 
       for (c = 0;  c < rel_n_records (categories); ++c)
         {
-          if (limit_category && str_cmp (limit_category, categories[c][0]))
+          if (limit_category && str_cmp (limit_category, rel_peek_str (categories, c, 0)))
             continue;
 
-          if (since_limits_opt && !assoc_ref (limit_categories, categories[c][0]))
+          if (since_limits_opt && !assoc_get_str_taking (limit_categories, rel_get_field (categories, c, 0)))
             continue;
 
           if (!omit_empty)
             {
-              safe_printfmt (1, "  %s\n", categories[c][0]);
+              safe_printfmt (1, "  %s\n", rel_peek_str (categories, c, 0));
               safe_flush (1);
               lim_free (0, last_category_printed);
-              last_category_printed = str_save (0, categories[c][0]);
+              last_category_printed = str_save (0, rel_peek_str (categories, c, 0));
             }
 
           if (omit_empty || (depth != category_depth))
             {
-              rel_table branches = 0;
+              rel_table branches = rel_table_nil;
               int b;
 
-              branches = arch_archive_branches (arch, categories[c][0]);
+              branches = arch_archive_branches (arch, rel_peek_str (categories, c, 0));
 
               for (b = 0;  b < rel_n_records (branches); ++b)
                 {
-                  if (limit_branch && str_cmp (limit_branch, branches[b][0]))
+                  if (limit_branch && str_cmp (limit_branch, rel_peek_str (branches, b, 0)))
                     continue;
 
-                  if (since_limits_opt && !assoc_ref (limit_branches, branches[b][0]))
+                  if (since_limits_opt && !assoc_get_str_taking (limit_branches, rel_get_field (branches, b, 0)))
                     continue;
 
                   if (!omit_empty)
                     {
-                      safe_printfmt (1, "    %s\n", branches[b][0]);
+                      safe_printfmt (1, "    %s\n", rel_peek_str (branches, b, 0));
                       safe_flush (1);
                       lim_free (0, last_branch_printed);
-                      last_branch_printed = str_save (0, branches[b][0]);
+                      last_branch_printed = str_save (0, rel_peek_str (branches, b, 0));
                     }
 
                   if (omit_empty || (depth != branch_depth))
                     {
-                      rel_table versions = 0;
+                      rel_table versions = rel_table_nil;
                       int v;
 
-                      versions = arch_archive_versions (arch, branches[b][0]);
+                      versions = arch_archive_versions (arch, rel_peek_str (branches, b, 0));
 
                       for (v = 0;  v < rel_n_records (versions); ++v)
                         {
-                          if (limit_version && str_cmp (limit_version, versions[v][0]))
+                          if (limit_version && str_cmp (limit_version, rel_peek_str (versions, v, 0)))
                             continue;
 
-                          if (since_limits_opt && !assoc_ref (limit_versions, versions[v][0]))
+                          if (since_limits_opt && !assoc_get_str_taking (limit_versions, rel_get_field (versions, v, 0)))
                             continue;
 
                           if (!omit_empty)
                             {
-			      safe_printfmt (1, "      %s\n", versions[v][0]);
+			      safe_printfmt (1, "      %s\n", rel_peek_str (versions, v, 0));
                               safe_flush (1);
                               lim_free (0, last_version_printed);
-                              last_version_printed = str_save (0, versions[v][0]);
+                              last_version_printed = str_save (0, rel_peek_str (versions, v, 0));
                             }
 
                           if (omit_empty || (depth != version_depth))
                             {
                               t_uchar * start_at_level = 0;
-                              rel_table revisions = 0;
+                              rel_table revisions = rel_table_nil;
                               int r;
                               int any_output = 0;
 
                               {
-                                t_uchar * since_limit = 0;
+                                const t_uchar * since_limit = 0;
 
-                                since_limit = assoc_ref (since_limits, versions[v][0]);
+                                since_limit = assoc_get_str_taking (since_limits, rel_get_field (versions, v, 0));
 
                                 if (since_limit)
                                   start_at_level = str_save (0, since_limit);
                               }
 
-                              revisions = arch_archive_revisions (arch, versions[v][0], 0);
+                              revisions = arch_archive_revisions (arch, rel_peek_str (versions, v, 0), 0);
 
                               if (reverse)
                                 {
                                   arch_sort_table_by_patch_level_field (1, revisions, 0);
-                                  if (revisions)
-                                    rel_add_records (&new_since_limits, rel_make_record (versions[v][0], revisions[0][0], 0), 0);
+                                  if (rel_n_records (revisions))
+                                    rel_add_records (&new_since_limits,
+                                                     rel_make_record_2_taking (rel_get_field (versions, v, 0),
+                                                                               rel_get_field (revisions, 0, 0)),
+                                                     0);
                                 }
                               else
                                 {
-                                  if (revisions)
-                                    rel_add_records (&new_since_limits, rel_make_record (versions[v][0], revisions[rel_n_records (revisions) - 1][0], 0), 0);
+                                  if (rel_n_records (revisions))
+                                    rel_add_records (&new_since_limits,
+                                                     rel_make_record_2_taking (rel_get_field (versions, v, 0),
+                                                                               rel_get_field (revisions, rel_n_records (revisions) - 1, 0)),
+                                                     0);
                                 }
 
                               for (r = 0; r < rel_n_records (revisions); ++r)
                                 {
                                   if (start_at_level)
                                     {
-                                      if (reverse && (0 <= arch_patch_lvl_cmp (start_at_level, revisions[r][0])))
+                                      if (reverse && (0 <= arch_patch_lvl_cmp (start_at_level, rel_peek_str (revisions, r, 0))))
                                         break;
 
-                                      if (!reverse && (0 <= arch_patch_lvl_cmp (start_at_level, revisions[r][0])))
+                                      if (!reverse && (0 <= arch_patch_lvl_cmp (start_at_level, rel_peek_str (revisions, r, 0))))
                                         continue;
                                     }
 
@@ -561,25 +577,25 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
                                       any_output = 1;
                                       if (omit_empty)
                                         {
-                                          if (str_cmp (last_category_printed, categories[c][0]))
+                                          if (str_cmp (last_category_printed, rel_peek_str (categories, c, 0)))
                                             {
-                                              safe_printfmt (1, "  %s\n", categories[c][0]);
+                                              safe_printfmt (1, "  %s\n", rel_peek_str (categories, c, 0));
                                               lim_free (0, last_category_printed);
-                                              last_category_printed = str_save (0, categories[c][0]);
+                                              last_category_printed = str_save (0, rel_peek_str (categories, c, 0));
                                             }
 
-                                          if (str_cmp (last_branch_printed, branches[b][0]))
+                                          if (str_cmp (last_branch_printed, rel_peek_str (branches, b, 0)))
                                             {
-                                              safe_printfmt (1, "    %s\n", branches[b][0]);
+                                              safe_printfmt (1, "    %s\n", rel_peek_str (branches, b, 0));
                                               lim_free (0, last_branch_printed);
-                                              last_branch_printed = str_save (0, branches[b][0]);
+                                              last_branch_printed = str_save (0, rel_peek_str (branches, b, 0));
                                             }
 
-                                          if (str_cmp (last_version_printed, versions[v][0]))
+                                          if (str_cmp (last_version_printed, rel_peek_str (versions, v, 0)))
                                             {
-                                              safe_printfmt (1, "      %s\n", versions[v][0]);
+                                              safe_printfmt (1, "      %s\n", rel_peek_str (versions, v, 0));
                                               lim_free (0, last_version_printed);
-                                              last_version_printed = str_save (0, versions[v][0]);
+                                              last_version_printed = str_save (0, rel_peek_str (versions, v, 0));
                                             }
                                         }
                                       if (wanted_headers)
@@ -588,32 +604,33 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
 
                                   if (!wanted_headers)
                                     {
-                                      if (!show_kind)
+                                      if (!show_kind && !show_cacherevs)
                                         {
                                           if (rel_n_records (revisions) == 1)
                                             {
-                                              safe_printfmt (1, "        %s\n", revisions[r][0]);
+                                              safe_printfmt (1, "        %s\n", rel_peek_str (revisions, r, 0));
                                             }
                                           else
                                             {
                                               int s;
                                               s = rel_n_records (revisions) - 1;
-                                              safe_printfmt (1, "        %s .. %s\n", revisions[r][0], revisions[s][0]);
+                                              safe_printfmt (1, "        %s .. %s\n", rel_peek_str (revisions, r, 0), rel_peek_str (revisions, s, 0));
                                               r = s;
                                             }
                                           }
                                       else
                                         {
-                                          /* show revision kinds */
+                                          /* show revision kinds and/or cacherevs */
                                           t_uchar * version_stem = 0;
                                           t_uchar * package_revision = 0;
                                           enum arch_revision_type this_type;
+					  int is_cached;
                                           int s;
 
-                                          version_stem = str_alloc_cat (0, versions[v][0], "--");
+                                          version_stem = str_alloc_cat (0, rel_peek_str (versions, v, 0), "--");
 
-                                          package_revision = str_alloc_cat (0, version_stem, revisions[r][0]);
-                                          arch_revision_type (&this_type, 0, arch, package_revision);
+                                          package_revision = str_alloc_cat (0, version_stem, rel_peek_str (revisions, r, 0));
+                                          arch_revision_type (&this_type, &is_cached, arch, package_revision);
 
                                           if (this_type == arch_continuation_revision)
                                             {
@@ -621,7 +638,12 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
 
                                               tagged = arch_get_continuation (arch, package_revision);
 
-                                              safe_printfmt (1, "        %s (tag of %s)\n", revisions[r][0], tagged);
+					      safe_printfmt (1, "        %s", rel_peek_str (revisions, r, 0));
+					      if (show_kind)
+						safe_printfmt (1, " (tag of %s)", tagged);
+					      if (show_cacherevs)
+						safe_printfmt (1, "%s", " [cached]");
+					      safe_printfmt (1, "\n");
 
                                               lim_free (0, tagged);
                                             }
@@ -630,30 +652,40 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
                                               for (s = r + 1; s < rel_n_records (revisions); ++s)
                                                 {
                                                   enum arch_revision_type s_type;
+						  int s_is_cached;
 
                                                   lim_free (0, package_revision);
-                                                  package_revision = str_alloc_cat (0, version_stem, revisions[s][0]);
-                                                  arch_revision_type (&s_type, 0, arch, package_revision);
+                                                  package_revision = str_alloc_cat (0, version_stem, rel_peek_str (revisions, s, 0));
+                                                  arch_revision_type (&s_type, &s_is_cached, arch, package_revision);
 
-                                                  if (s_type != this_type)
+						  if ((show_cacherevs && (s_is_cached != is_cached))
+						      || (show_kind && (s_type != this_type)))
                                                     break;
                                                 }
                                               --s;
 
                                               if (s == r)
                                                 {
-                                                  safe_printfmt (1, "          %s ", revisions[r][0]);
+                                                  safe_printfmt (1, "          %s", rel_peek_str (revisions, r, 0));
                                                 }
                                               else
                                                 {
-                                                  safe_printfmt (1, "          %s .. %s ", revisions[r][0], revisions[s][0]);
+                                                  safe_printfmt (1, "          %s .. %s", rel_peek_str (revisions, r, 0), rel_peek_str (revisions, s, 0));
                                                 }
 
-                                              if (this_type == arch_import_revision)
-                                                safe_printfmt (1, "(initial import)\n");
-                                              else
-                                                safe_printfmt (1, "(simple changeset)\n");
-
+					      if (show_kind)
+						{
+						  safe_printfmt (1, " ");
+						  if (this_type == arch_import_revision)
+						    safe_printfmt (1, "(initial import)");
+						  else
+						    safe_printfmt (1, "(simple changeset)");
+						}
+						  
+					      if (show_cacherevs && is_cached)
+						safe_printfmt (1, "%s", " [cached]");
+					      safe_printfmt (1, "\n");
+					      
                                               r = s;
                                             }
 
@@ -670,41 +702,49 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
                                       t_uchar * log = 0;
                                       assoc_table headers = 0;
 
-                                      rev = str_alloc_cat_many (0, versions[v][0], "--", revisions[r][0], str_end);
+                                      rev = str_alloc_cat_many (0, rel_peek_str (versions, v, 0), "--", rel_peek_str (revisions, r, 0), str_end);
 				      fqrev = arch_fully_qualify (archive, rev);
-                                      if (!show_kind)
+                                      if (!show_kind && !show_cacherevs)
                                         {
-                                          safe_printfmt (1, "        %s\n", full ? fqrev : revisions[r][0]);
+                                          safe_printfmt (1, "        %s\n", full ? fqrev : rel_peek_str (revisions, r, 0));
                                         }
                                       else
                                         {
                                           enum arch_revision_type type;
+					  int is_cached;
 
-                                          arch_revision_type (&type, 0, arch, rev);
+                                          arch_revision_type (&type, &is_cached, arch, rev);
 
-                                          switch (type)
-                                            {
-                                            case arch_import_revision:
-                                              {
-                                                safe_printfmt (1, "        %s    (initial import)\n", full ? fqrev : revisions[r][0]);
-                                                break;
-                                              }
-                                            case arch_simple_revision:
-                                              {
-                                                safe_printfmt (1, "        %s    (simple changeset)\n", full ? fqrev : revisions[r][0]);
-                                                break;
-                                              }
-                                            case arch_continuation_revision:
-                                              {
-                                                t_uchar * continuation_of = 0;
+					  safe_printfmt (1, "        %s   ", full ? fqrev : rel_peek_str (revisions, r, 0));
+					  if (show_kind)
+					    {
+					      switch (type)
+						{
+						case arch_import_revision:
+						  {
+						    safe_printfmt (1, "(initial import)");
+						    break;
+						  }
+						case arch_simple_revision:
+						  {
+						    safe_printfmt (1, "(simple changeset)");
+						    break;
+						  }
+						case arch_continuation_revision:
+						  {
+						    t_uchar * continuation_of = 0;
 
-                                                continuation_of = arch_get_continuation (arch, rev);
-                                                safe_printfmt (1, "        %s    (tag revision of %s)\n", full ? fqrev : revisions[r][0], continuation_of);
+						    continuation_of = arch_get_continuation (arch, rev);
+						    safe_printfmt (1, "(tag revision of %s)", continuation_of);
 
-                                                lim_free (0, continuation_of);
-                                                break;
-                                              }
-                                            }
+						    lim_free (0, continuation_of);
+						    break;
+						  }
+						}
+					    }
+					  if (show_cacherevs && is_cached)
+					    safe_printfmt (1, "%s", " [cached]");
+					  safe_printfmt (1, "\n");
                                         }
 
                                       log = arch_archive_log (arch, rev);
@@ -749,9 +789,9 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
 
     if (snap)
       {
-        rel_table unchanged = 0;
-        rel_table changed = 0;
-        rel_table all = 0;
+        rel_table unchanged = rel_table_nil;
+        rel_table changed = rel_table_nil;
+        rel_table all = rel_table_nil;
         int out_fd;
         int x;
 
@@ -768,7 +808,7 @@ arch_cmd_abrowse (t_uchar * program_name, int argc, char * argv[])
 
         for (x = 0; x < rel_n_records (all); ++x)
           {
-            safe_printfmt (out_fd, "%s--%s\n", all[x][0], all[x][1]);
+            safe_printfmt (out_fd, "%s--%s\n", rel_peek_str (all, x, 0), rel_peek_str (all, x, 1));
           }
 
         safe_close (out_fd);
