@@ -1,7 +1,7 @@
-/* file-diffs.c
+/* cmd-file-find.c
  *
  ****************************************************************
- * Copyright (C) 2003 Tom Lord
+ * Copyright (C) 2003, 2004 Tom Lord
  *
  * See the file "COPYING" for further information about
  * the copyright and warranty status of this work.
@@ -21,7 +21,7 @@
 #include "tla/libarch/namespace.h"
 #include "tla/libfsutils/dir-as-cwd.h"
 #include "tla/libarch/cmd.h"
-#include "tla/libarch/cmd-file-diffs.h"
+#include "tla/libarch/cmd-file-diff.h"
 
 
 
@@ -50,7 +50,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
   OP (opt_archive, "A", "archive", 1, \
       "Override `my-default-archive'") \
   OP (opt_new_file, "N", "new-file", 0, \
-      "Print missing file as `/dev/null'")
+      "Print missing file as `/dev/null'") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 t_uchar arch_cmd_file_find_help[] = ("find given version of file\n"
                                       "Print location of file corresponding to FILE in a cached copy of REVISION\n"
@@ -79,6 +81,7 @@ arch_cmd_file_find (t_uchar * program_name, int argc, char * argv[])
   int o;
   struct opt_parsed * option;
   t_uchar * default_archive;
+  int escape_classes = arch_escape_classes;
   int new_is_null = 0;
   int status = 2;
 
@@ -117,6 +120,11 @@ arch_cmd_file_find (t_uchar * program_name, int argc, char * argv[])
             new_is_null = 1;
             break;
           }
+	case opt_unescaped:
+	  {
+	    escape_classes = 0;
+	    break;
+	  }
         }
     }
 
@@ -192,8 +200,7 @@ arch_cmd_file_find (t_uchar * program_name, int argc, char * argv[])
     archive = arch_parse_package_name (arch_ret_archive, default_archive, revspec);
     if (!archive)
       {
-        safe_printfmt (2, "%s: no archive specified for file-diffs\n",
-                       argv[0]);
+        safe_printfmt (2, "%s: no archive specified\n", argv[0]);
         exit (2);
       }
 
@@ -216,7 +223,7 @@ arch_cmd_file_find (t_uchar * program_name, int argc, char * argv[])
       }
 
 
-    status = arch_file_get_or_diff (1, tree_root, mod_loc, archive, revision, 0, new_is_null);
+    status = arch_file_get_or_diff (1, tree_root, mod_loc, archive, revision, 0, new_is_null, escape_classes);
 
 
     lim_free (0, filedir);
