@@ -228,10 +228,10 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
 
     if (library)
       {
-        rel_table library_path = 0;
+        rel_table library_path = rel_table_nil;
 
         library_path = arch_my_library_path (arch_library_path_search_order);
-        if (!library_path)
+        if (!rel_n_records (library_path))
           {
             safe_printfmt (2, "%s: no default library (try tla my-revision-library -H)\n", argv[0]);
             exit (2);
@@ -250,11 +250,12 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
 
     archive = arch_parse_package_name (arch_ret_archive, default_archive, revision_spec);
     arch = arch_archive_connect (archive, 0);
+    arch_check_arch (arch);
 
     if (arch_valid_package_name (revision_spec, arch_maybe_archive, arch_req_package, 0))
       {
         t_uchar * package = 0;
-        rel_table versions = 0;
+        rel_table versions = rel_table_nil;
 
         package = arch_parse_package_name (arch_ret_package, 0, revision_spec);
 
@@ -262,7 +263,7 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
 
         versions = arch_archive_versions (arch, package);
         arch_sort_table_by_name_field (1, versions, 0);
-        if (!versions)
+        if (!rel_n_records (versions))
           {
             safe_printfmt (2, "%s: package has no versions (%s)\n",
                            argv[0], revision_spec);
@@ -270,7 +271,7 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
           }
 
         lim_free (0, revision_spec);
-        revision_spec = str_save (0, versions[0][0]);
+        revision_spec = str_save (0, rel_peek_str (versions, 0, 0));
 
         lim_free (0, package);
         rel_free_table (versions);
@@ -279,21 +280,21 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
 
     if (arch_valid_package_name (revision_spec, arch_maybe_archive, arch_req_version, 0))
       {
-        rel_table revisions = 0;
+        rel_table revisions = rel_table_nil;
 
 	arch_check_for (arch, arch_req_version, revision_spec);
         version = arch_parse_package_name (arch_ret_package_version, 0, revision_spec);
 
         revisions = arch_archive_revisions (arch, version, 1);
         arch_sort_table_by_name_field (1, revisions, 0);
-        if (!revisions)
+        if (!rel_n_records (revisions))
           {
             safe_printfmt (2, "%s: no revisions in version (%s/%s)\n",
                            argv[0], archive, revision_spec);
             exit (2);
           }
 
-        revision = arch_parse_package_name (arch_ret_non_archive, 0, revisions[0][0]);
+        revision = arch_parse_package_name (arch_ret_non_archive, 0, rel_peek_str (revisions, 0, 0));
 
         rel_free_table (revisions);
       }
@@ -373,14 +374,20 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
     if (hardlinks)
       {
         t_uchar * revision_dir = 0;
-        rel_table index = 0;
+        rel_table index = rel_table_nil;
+        int full_meta;
 
         index = arch_library_index (archive, revision);
         rel_sort_table_by_field (0, index, 0);
 
         safe_printfmt (1, "* hard linking to library\n");
         safe_flush (1);
-        build_partial_link_tree (arch_library_find (0, archive, revision, 1), tmp_dir, index);
+
+        revision_dir = arch_library_find (rel_table_nil, archive, revision, 1);
+        invariant (0 != revision_dir);
+        full_meta = arch_tree_has_meta_flag (revision_dir);
+        
+        build_partial_link_tree (arch_library_find (rel_table_nil, archive, revision, 1), tmp_dir, index, full_meta);
         arch_snap_inode_sig (tmp_dir, archive, revision);
 
         lim_free (0, revision_dir);
