@@ -21,22 +21,23 @@
 
 
 
-
-static void rbrowse_save_since (t_uchar *filename, int snap_force, rel_table since_table);
-static rel_table rbrowse_load_since (t_uchar *filename);
-static void rbrowse_print_logs (rel_table patch_output, int summarized_headers);
-static int rbrowse_since_has_newest ( t_uchar *revision, rel_table saved_since);
-
-static rel_table rbrowse_patchlog ( struct arch_archive * arch,
-                            regex_t patch_regex_needle,
-                            t_uchar *version,
-                            rel_table saved_since);
-
-void rbrowse_build_since_table (struct arch_archive *arch, 
-                                   t_uchar *version,
-                                   rel_table * since_output);
-
-static t_uchar * rbrowse_revision_range (t_uchar *latest_revision);
+/* __STDC__ prototypes for static functions */
+static void rbrowse_print_logs (rel_table patch_output,
+                                int summarized_headers);
+static rel_table rbrowse_patchlog (struct arch_archive * arch,
+                                   regex_t patch_regex_needle,
+                                   const t_uchar * version,
+                                   rel_table saved_since);
+static int rbrowse_since_has_newest (t_uchar * revision,
+                                     rel_table saved_since);
+static t_uchar * rbrowse_revision_range (const t_uchar * latest_revision);
+static void rbrowse_build_since_table (struct arch_archive *arch, 
+                                       const t_uchar *version,
+                                       rel_table * since_output);
+static rel_table rbrowse_load_since (const t_uchar *filename);
+static void rbrowse_save_since (const t_uchar *filename,
+                                int snap_force,
+                                rel_table since_table);
 
 
 
@@ -111,7 +112,9 @@ static struct opt_desc opts[] =
 
 
 int
-arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
+arch_cmd_rbrowse (t_uchar * program_name,
+                  int argc,
+                  char * argv[])
 {
   int o;
   int w;
@@ -121,18 +124,18 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
   int want_logs = 0;
   int unhide_sealed = 0;
   int log_reverse=0;
-  struct opt_parsed * option = NULL;
+  struct opt_parsed * option = 0;
   regex_t patch_regex_needle;
   regex_t limit_regex_needle;
   t_uchar * errname;
-  t_uchar * snap_file = NULL;
-  t_uchar * since_filename = NULL;
-  t_uchar * default_archive = NULL;
-  t_uchar * patch_regex = NULL;
-  t_uchar * limit_regex = NULL;
-  rel_table since_output = NULL;
-  rel_table saved_since = NULL;
-  rel_table archive_list = 0;
+  t_uchar * snap_file = 0;
+  t_uchar * since_filename = 0;
+  t_uchar * default_archive = 0;
+  t_uchar * patch_regex = 0;
+  t_uchar * limit_regex = 0;
+  rel_table since_output = rel_table_nil;
+  rel_table saved_since = rel_table_nil;
+  rel_table archive_list = rel_table_nil;
 
 
   errname = argv[0];
@@ -276,7 +279,7 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
 
   if (! all_archives)
     {
-      rel_add_records (&archive_list, rel_make_record (default_archive, 0), 0);
+      rel_add_records (&archive_list, rel_singleton_record_taking (rel_make_field_str (default_archive)), 0);
     }
   else
     {
@@ -290,30 +293,30 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
       int z;
       int shown_archive = 0;
       struct arch_archive * arch = 0;
-      rel_table categories = 0;
+      rel_table categories = rel_table_nil;
 
 
-      if (str_cmp("-MIRROR", str_chr_rindex(archive_list[w][0], '-')) == 0 ||
-          str_cmp("-SOURCE", str_chr_rindex(archive_list[w][0], '-')) == 0 )
+      if (str_cmp("-MIRROR", str_chr_rindex(rel_peek_str (archive_list, w, 0), '-')) == 0 ||
+          str_cmp("-SOURCE", str_chr_rindex(rel_peek_str (archive_list, w, 0), '-')) == 0 )
         continue;
 
-      arch = arch_archive_connect(archive_list[w][0], 0);
+      arch = arch_archive_connect(rel_peek_str (archive_list, w, 0), 0);
 
       categories = arch_archive_categories(arch);
 
       for (x = 0; x < rel_n_records (categories); ++x)
         {
           int shown_category = 0;
-          rel_table branches = 0;
+          rel_table branches = rel_table_nil;
 
-          branches = arch_archive_branches(arch, categories[x][0]);
+          branches = arch_archive_branches(arch, rel_peek_str (categories, x, 0));
 
           for (y = 0; y < rel_n_records (branches); ++y)
             {
               int shown_branch = 0;
-              rel_table versions = 0;
+              rel_table versions = rel_table_nil;
 
-              versions = arch_archive_versions(arch, branches[y][0]);
+              versions = arch_archive_versions(arch, rel_peek_str (branches, y, 0));
 
               for (z = 0; z < rel_n_records (versions); ++z)
                 {
@@ -325,10 +328,10 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
                   int sealed_branch;
                   t_uchar * patch;
                   t_uchar * only_patch;
-                  t_uchar * patch_line = NULL;
+                  t_uchar * patch_line = 0;
                   rel_table patch_output;
 
-                  patch = arch_archive_latest_revision(arch, versions[z][0],1);
+                  patch = arch_archive_latest_revision(arch, rel_peek_str (versions, z, 0),1);
 
                   if (!patch)
                     {
@@ -351,10 +354,10 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
                     }
                   
                   if (snap_file)
-                    rbrowse_build_since_table(arch, versions[z][0], &since_output);
+                    rbrowse_build_since_table (arch, rel_peek_str (versions, z, 0), &since_output);
                   
 
-                  if (regexec (&limit_regex_needle, versions[z][0], 0, 0, 0) == 0)
+                  if (regexec (&limit_regex_needle, rel_peek_str (versions, z, 0), 0, 0, 0) == 0)
                     version_show = 1;
                         
                   needshow = needshow || since_changed;
@@ -362,7 +365,7 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
 
                   if (needshow && want_logs)
                     {
-                      patch_output = rbrowse_patchlog(arch, patch_regex_needle, versions[z][0], saved_since);
+                      patch_output = rbrowse_patchlog(arch, patch_regex_needle, rel_peek_str (versions, z, 0), saved_since);
                       have_patches = !! rel_n_records(patch_output);
                       needshow = needshow && have_patches;
                     }
@@ -380,20 +383,20 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
                       if (!shown_archive)
                         {
                           shown_archive = 1;
-                          safe_printfmt (1,"%s\n", archive_list[w][0]);
+                          safe_printfmt (1,"%s\n", rel_peek_str (archive_list, w, 0));
                         }
                       if (!shown_category)
                         {
                           shown_category = 1;
-                          safe_printfmt (1, "   %s\n", categories[x][0]);
+                          safe_printfmt (1, "   %s\n", rel_peek_str (categories, x, 0));
                         }
                       if (!shown_branch)
                         {
                           shown_branch = 1;
-                          safe_printfmt(1, "      %s\n", branches[y][0]);
+                          safe_printfmt(1, "      %s\n", rel_peek_str (branches, y, 0));
                         }
 
-                      safe_printfmt(1, "         %s\n", versions[z][0]);
+                      safe_printfmt(1, "         %s\n", rel_peek_str (versions, z, 0));
                       
                       patch_line =  rbrowse_revision_range (only_patch);
                       safe_printfmt(1, "             %s\n\n", patch_line);
@@ -426,7 +429,7 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
 
   if (snap_file)
     {
-      if (! since_output)
+      if (!rel_n_records (since_output))
         panic("Nothing to Snap!");
       else
         {
@@ -437,8 +440,10 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
   return 0;
 }
 
+
 static void
-rbrowse_print_logs (rel_table patch_output, int summarized_headers)
+rbrowse_print_logs (rel_table patch_output,
+                    int summarized_headers)
 {
   int x;
   
@@ -451,8 +456,8 @@ rbrowse_print_logs (rel_table patch_output, int summarized_headers)
       t_uchar * patch_name;
       
 
-      arch_parse_log (0, &headers, NULL, patch_output[x][0]);
-      this_revision = str_save (0, assoc_ref (headers, "revision"));
+      arch_parse_log (0, &headers, 0, rel_peek_str (patch_output, x, 0));
+      this_revision = str_save (0, assoc_get_str_taking (headers, rel_make_field_str ("revision")));
       patch_name = arch_parse_package_name (arch_ret_patch_level, 0, this_revision);
 
       if (summarized_headers)
@@ -468,11 +473,15 @@ rbrowse_print_logs (rel_table patch_output, int summarized_headers)
     }
 }
 
+
 static rel_table
-rbrowse_patchlog( struct arch_archive * arch, regex_t patch_regex_needle, t_uchar *version, rel_table saved_since)
+rbrowse_patchlog (struct arch_archive * arch,
+                  regex_t patch_regex_needle,
+                  const t_uchar * version,
+                  rel_table saved_since)
 {
   rel_table revisions;
-  rel_table patch_output = NULL;
+  rel_table patch_output = rel_table_nil;
   int saw_since;
   int x=0;
 
@@ -485,14 +494,14 @@ rbrowse_patchlog( struct arch_archive * arch, regex_t patch_regex_needle, t_ucha
       int y;
       int missing_in_since = 1;
       t_uchar *revisions_version;
-      revisions_version = arch_parse_package_name (arch_ret_package_version, 0, revisions[x][0]);
+      revisions_version = arch_parse_package_name (arch_ret_package_version, 0, rel_peek_str (revisions, x, 0));
 
       /* First, see if our revision exists in the since file */
       for (y=0; y < rel_n_records(saved_since); ++y)
         {
           t_uchar *since_version;
 
-          since_version = arch_parse_package_name (arch_ret_package_version, 0, saved_since[y][0]);
+          since_version = arch_parse_package_name (arch_ret_package_version, 0, rel_peek_str (saved_since, y, 0));
 
           if (str_cmp(revisions_version, since_version) == 0)
             missing_in_since = 0;
@@ -505,18 +514,18 @@ rbrowse_patchlog( struct arch_archive * arch, regex_t patch_regex_needle, t_ucha
           t_uchar * log;
           t_uchar * revision;
           t_uchar * summary;
-          assoc_table headers = NULL;
+          assoc_table headers = 0;
 
-          revision = str_chr_index (revisions[x][0], '/');
+          revision = str_chr_index (rel_peek_str (revisions, x, 0), '/');
           revision++;
 
           log = arch_archive_log(arch, revision);
-          arch_parse_log (0, &headers, NULL, log);
-          summary = str_save (0, assoc_ref (headers, "summary"));
+          arch_parse_log (0, &headers, 0, log);
+          summary = str_save (0, assoc_get_str_taking (headers, rel_make_field_str ("summary")));
           
           if (regexec (&patch_regex_needle, summary, 0, 0, 0) == 0)
             {
-              rel_add_records (&patch_output, rel_make_record (log, 0), 0);
+              rel_add_records (&patch_output, rel_singleton_record_taking (rel_make_field_str (log)), 0);
             }
 
           lim_free(0, summary);
@@ -528,7 +537,7 @@ rbrowse_patchlog( struct arch_archive * arch, regex_t patch_regex_needle, t_ucha
           int z;
           for (z = 0 ; z < rel_n_records(saved_since); z++)
             {
-              if ( str_cmp(revisions[x][0], saved_since[z][0]) == 0)
+              if ( str_cmp(rel_peek_str (revisions, x, 0), rel_peek_str (saved_since, z, 0)) == 0)
                 saw_since = 1;
             }
         }
@@ -538,22 +547,25 @@ rbrowse_patchlog( struct arch_archive * arch, regex_t patch_regex_needle, t_ucha
   return patch_output;
 }
 
+
 static int
-rbrowse_since_has_newest ( t_uchar *revision, rel_table saved_since)
+rbrowse_since_has_newest (t_uchar * revision,
+                          rel_table saved_since)
 {
   int exists=0;
   int x;
 
   for (x=0; x < rel_n_records (saved_since); ++x)
     {
-      if (str_cmp (revision, saved_since[x][0]) == 0)
+      if (str_cmp (revision, rel_peek_str (saved_since, x, 0)) == 0)
         exists = 1;
     }
   return exists;
 }
 
+
 static t_uchar *
-rbrowse_revision_range (t_uchar * latest_revision)
+rbrowse_revision_range (const t_uchar * latest_revision)
 {
   t_uchar *retval;
 
@@ -571,21 +583,23 @@ rbrowse_revision_range (t_uchar * latest_revision)
   return retval;
 }
 
-void 
+
+static void 
 rbrowse_build_since_table (struct arch_archive *arch, 
-                                   t_uchar *version,
-                                   rel_table * since_output)
+                           const t_uchar *version,
+                           rel_table * since_output)
 {
   t_uchar *latest_revision;
   
   latest_revision = arch_archive_latest_revision (arch, version, 1);
-  rel_add_records ( since_output, rel_make_record (latest_revision, 0),0);
+  rel_add_records (since_output, rel_singleton_record_taking (rel_make_field_str (latest_revision)),0);
   lim_free(0, latest_revision);
 
 }
 
+
 static rel_table
-rbrowse_load_since (t_uchar *filename)
+rbrowse_load_since (const t_uchar *filename)
 {
   int in_fd;
   rel_table since_table;
@@ -597,9 +611,9 @@ rbrowse_load_since (t_uchar *filename)
   
   for (x=0; x < rel_n_records( since_table ); ++x)
     {
-      if (! arch_valid_package_name( since_table[x][0], arch_maybe_archive, arch_req_version, 1))
+      if (! arch_valid_package_name( rel_peek_str (since_table, x, 0), arch_maybe_archive, arch_req_version, 1))
         {
-          safe_printfmt (2, "illegal line ('%s') in since file '%s'\n", since_table[x][0], filename);
+          safe_printfmt (2, "illegal line ('%s') in since file '%s'\n", rel_peek_str (since_table, x, 0), filename);
           exit(1);
         }
     }
@@ -609,8 +623,11 @@ rbrowse_load_since (t_uchar *filename)
   return since_table;
 }
 
+
 static void
-rbrowse_save_since (t_uchar *filename,int snap_force, rel_table since_table)
+rbrowse_save_since (const t_uchar *filename,
+                    int snap_force,
+                    rel_table since_table)
 {
   int out_fd;
   int x;
@@ -620,7 +637,7 @@ rbrowse_save_since (t_uchar *filename,int snap_force, rel_table since_table)
 
   for (x = 0; x < rel_n_records(since_table); ++x)
     {
-      safe_printfmt(out_fd, "%s\n", since_table[x][0]);
+      safe_printfmt(out_fd, "%s\n", rel_peek_str (since_table, x, 0));
     }
   safe_close (out_fd);
 }
