@@ -312,13 +312,14 @@ arch_cmd_replay (t_uchar * program_name, int argc, char * argv[])
     if (dest)
       {
         rel_table to_copy = rel_table_nil;
+        const int full_meta = arch_tree_has_meta_flag (tree_root);
 
         safe_printfmt (1, "* copying target tree\n");
         safe_flush (1);
 
         to_copy = arch_source_inventory (tree_root, 1, 1, 0);
         safe_mkdir (dest, 0777);
-        copy_file_list (dest, tree_root, to_copy);
+        copy_file_list (dest, tree_root, to_copy, full_meta);
 
         rel_free_table (to_copy);
       }
