@@ -39,6 +39,9 @@
 
 
 /* __STDC__ prototypes for static functions */
+
+
+
 static t_uchar * arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
                                                 int chatter_fd,
                                                 t_uchar ** cooked_log_ret,
@@ -47,13 +50,22 @@ static t_uchar * arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
                                                 struct arch_archive * arch,
                                                 t_uchar * revision,
                                                 t_uchar * prev_level,
-                                                rel_table file_list);
+                                                rel_table file_list,
+                                                int escape_classes);
 static rel_table pick_non_control (rel_table table);
 static rel_table pick_patch_logs (rel_table table);
 static void arch_commit_mid_commit (t_uchar * tree_root, t_uchar * cooked_log);
-static void arch_finish_commit (int chatter_fd, t_uchar * tree_root, t_uchar * archive, t_uchar * prev_revision, t_uchar * revision, t_uchar * changeset, t_uchar * changelog_loc);
+static void arch_finish_commit (int chatter_fd,
+                                t_uchar * tree_root,
+                                t_uchar * archive,
+                                t_uchar * prev_revision,
+                                t_uchar * revision,
+                                t_uchar * changeset,
+                                t_uchar * changelog_loc,
+                                int escape_classes,
+                                int full_meta);
 static void arch_commit_failed (t_uchar * tree_root, t_uchar * archive, t_uchar * revision, t_uchar * changeset);
-static void commit_make_changeset_callback (void * vfd, char * fmt, va_list ap);
+static void commit_make_changeset_callback (void * vfd, const char * fmt, va_list ap);
 
 
 
@@ -63,8 +75,10 @@ arch_commit (int chatter_fd,
              t_uchar * revision,
              t_uchar * tree_root,
              t_uchar * raw_log,
-             rel_table file_list)
+             rel_table file_list,
+             int escape_classes)
 {
+  int full_meta;
   t_uchar * errstr;
   t_uchar * version = 0;
   t_uchar * this_level = 0;
@@ -79,12 +93,14 @@ arch_commit (int chatter_fd,
 
   invariant (!!raw_log);
 
+  full_meta = arch_tree_has_meta_flag (tree_root);
+
   version = arch_parse_package_name (arch_ret_package_version, 0, revision);
   this_level = arch_parse_package_name (arch_ret_patch_level, 0, revision);
   prev_revision = arch_previous_revision (arch, revision);
   prev_level = arch_parse_package_name (arch_ret_patch_level, 0, prev_revision);
 
-  changeset_path = arch_prepare_commit_changeset (&changelog_loc, chatter_fd, &cooked_log, tree_root, raw_log, arch, revision, prev_level, file_list);
+  changeset_path = arch_prepare_commit_changeset (&changelog_loc, chatter_fd, &cooked_log, tree_root, raw_log, arch, revision, prev_level, file_list, escape_classes);
 
   invariant (!!prev_level);
 
@@ -139,7 +155,7 @@ arch_commit (int chatter_fd,
       exit (2);
     }
 
-  arch_finish_commit (chatter_fd, tree_root, arch->name, prev_revision, revision, changeset_path, changelog_loc);
+  arch_finish_commit (chatter_fd, tree_root, arch->name, prev_revision, revision, changeset_path, changelog_loc, escape_classes, full_meta);
 
   if (rel_n_records (file_list) == 0) 
     arch_snap_inode_sig (tree_root, arch->name, revision);
@@ -173,7 +189,8 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
                                struct arch_archive * arch,
                                t_uchar * revision,
                                t_uchar * prev_level,
-                               rel_table file_list)
+                               rel_table file_list,
+                               int escape_classes)
 {
   t_uchar * archive = arch->name;
   t_uchar * tmp_stem = 0;
@@ -191,7 +208,7 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
   t_uchar * changelog_id = 0;
   t_uchar * changelog_orig_loc = 0;
   t_uchar * changelog_mod_loc = 0;
-  struct arch_changeset_report csr = {0, };
+  struct arch_changeset_report csr = {rel_table_nil, };
   int changelog_diffs_fd = -1;
   int changelog_add_fd = -1;
   t_uchar * new_log_id = 0;
@@ -240,18 +257,18 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
       make_report.thunk = (void *)(long)chatter_fd;
     }
 
-  if (!file_list)
+  if (!rel_n_records (file_list))
     {
       assoc_table inode_shortcut = 0;
 
       arch_read_inode_sig (0, &inode_shortcut, tree_root, archive, prev_revision);
 
-      arch_make_changeset (&make_report, prev_rev_path, tree_root, changeset_path, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0);
+      arch_make_changeset (&make_report, prev_rev_path, tree_root, changeset_path, arch_unspecified_id_tagging, arch_inventory_unrecognized, rel_table_nil, inode_shortcut, 0, escape_classes);
 
       free_assoc_table (inode_shortcut);
     }
   else
-    arch_make_files_changeset (&make_report, changeset_path, file_list, prev_rev_path, tree_root, arch_unspecified_id_tagging, arch_inventory_unrecognized);
+    arch_make_files_changeset (&make_report, changeset_path, file_list, prev_rev_path, tree_root, arch_unspecified_id_tagging, arch_inventory_unrecognized, escape_classes);
 
   /****************************************************************
    * Look for a Changelog for the version we're committing to.
@@ -261,21 +278,32 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
   changelog_x_id = str_alloc_cat (0, "x", changelog_id_suffix);
   changelog_i_id = str_alloc_cat (0, "i", changelog_id_suffix);
 
-  changelog_mod_loc = assoc_ref (make_report.mod_file_loc_of, changelog_x_id);
-  if (changelog_mod_loc)
-    {
-      changelog_mod_loc = str_save (0, changelog_mod_loc);
-      changelog_id = str_save (0, changelog_x_id);
-    }
-  else
-    {
-      changelog_mod_loc = assoc_ref (make_report.mod_file_loc_of, changelog_i_id);
-      if (changelog_mod_loc)
-        {
-          changelog_mod_loc = str_save (0, changelog_mod_loc);
-          changelog_id = str_save (0, changelog_i_id);
-        }
-    }
+  {
+    const t_uchar * tmp;
+
+    tmp = assoc_get_str_taking (make_report.mod_file_loc_of, rel_make_field_str (changelog_x_id));
+    if (tmp)
+      {
+        changelog_mod_loc = str_save (0, tmp);
+        changelog_id = str_save (0, changelog_x_id);
+      }
+    else
+      {
+        const t_uchar * tmp;
+
+        tmp = assoc_get_str_taking (make_report.mod_file_loc_of, rel_make_field_str (changelog_i_id));
+        if (tmp)
+          {
+            changelog_mod_loc = str_save (0, tmp);
+            changelog_id = str_save (0, changelog_i_id);
+          }
+        else
+          {
+            changelog_mod_loc = 0;
+            changelog_id = 0;
+          }
+      }
+  }
 
   if (changelog_mod_loc)
     {
@@ -284,9 +312,15 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
           *changelog_loc_ret = str_save (0, changelog_mod_loc);
         }
 
-      changelog_orig_loc = assoc_ref (make_report.orig_file_loc_of, changelog_id);
-      if (changelog_orig_loc)
-        changelog_orig_loc = str_save (0, changelog_orig_loc);
+      {
+        const t_uchar * tmp;
+        
+        tmp = assoc_get_str_taking (make_report.orig_file_loc_of, rel_make_field_str (changelog_id));
+        if (tmp)
+          changelog_orig_loc = str_save (0, tmp);
+        else
+          changelog_orig_loc = 0;
+      }
     }
 
   /* post-condition:
@@ -327,7 +361,7 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
 
   if (changelog_mod_loc && changelog_orig_loc)
     {
-      changelog_diffs_fd = arch_changeset_add_diffs (&csr, &make_report, changeset_path, changelog_mod_loc, changelog_orig_loc, changelog_id);
+      changelog_diffs_fd = arch_changeset_add_diffs (&csr, &make_report, changeset_path, changelog_orig_loc, changelog_mod_loc, changelog_id);
     }
   else if (changelog_mod_loc)
     {
@@ -369,13 +403,13 @@ arch_prepare_commit_changeset (t_uchar ** changelog_loc_ret,
      */
 
     {
-      rel_table new_files = 0;
-      rel_table removed_files = 0;
-      rel_table new_directories = 0;
-      rel_table removed_directories = 0;
-      rel_table modified_files = 0;
-      rel_table new_patches = 0;
-      rel_table removed_patches = 0;
+      rel_table new_files = rel_table_nil;
+      rel_table removed_files = rel_table_nil;
+      rel_table new_directories = rel_table_nil;
+      rel_table removed_directories = rel_table_nil;
+      rel_table modified_files = rel_table_nil;
+      rel_table new_patches = rel_table_nil;
+      rel_table removed_patches = rel_table_nil;
 
       new_files = pick_non_control (csr.added_files);
       rel_append_x (&new_files, csr.added_symlinks);
@@ -547,16 +581,16 @@ static rel_table
 pick_non_control (rel_table table)
 {
   int x;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
 
   for (x = 0; x < rel_n_records (table); ++x)
     {
-      t_uchar * id;
+      const t_uchar * id;
 
-      id = table[x][1];
+      id = rel_peek_str (table, x, 1);
 
       if (str_cmp_prefix ("A_", id))
-        rel_add_records (&answer, rel_copy_record (table[x]), 0);
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (table, x)), 0);
     }
 
   return answer;
@@ -567,7 +601,7 @@ static rel_table
 pick_patch_logs (rel_table table)
 {
   int x;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
 
   for (x = 0; x < rel_n_records (table); ++x)
     {
@@ -580,7 +614,7 @@ pick_patch_logs (rel_table table)
       t_uchar * revision = 0;
       t_uchar * fqrev = 0;
 
-      f = str_save (0, table[x][0]);
+      f = str_save (0, rel_peek_str (table, x, 0));
       t = file_name_tail (0, f);
       d = file_name_directory_file (0, f);
 
@@ -631,7 +665,11 @@ pick_patch_logs (rel_table table)
                                 {
                                   revision = str_alloc_cat_many (0, version, "--", level, str_end);
                                   fqrev = arch_fully_qualify (archive, revision);
-                                  rel_add_records (&answer, rel_make_record (fqrev, table[x][1], table[x][2], 0), 0);
+                                  rel_add_records (&answer,
+                                                   rel_make_record_3_taking (rel_make_field_str (fqrev),
+                                                                             rel_get_field (table, x, 1),
+                                                                             rel_get_field (table, x, 2)),
+                                                   0);
                                 }
                             }
                         }
@@ -664,7 +702,15 @@ arch_commit_mid_commit (t_uchar * tree_root, t_uchar * cooked_log)
 
 
 static void
-arch_finish_commit (int chatter_fd, t_uchar * tree_root, t_uchar * archive, t_uchar * prev_revision, t_uchar * revision, t_uchar * changeset, t_uchar * changelog_loc)
+arch_finish_commit (int chatter_fd,
+                    t_uchar * tree_root,
+                    t_uchar * archive,
+                    t_uchar * prev_revision,
+                    t_uchar * revision,
+                    t_uchar * changeset,
+                    t_uchar * changelog_loc,
+                    int escape_classes,
+                    int full_meta)
 {
   t_uchar * dir = 0;
   t_uchar * dir_tail = 0;
@@ -675,7 +721,7 @@ arch_finish_commit (int chatter_fd, t_uchar * tree_root, t_uchar * archive, t_uc
   dir_tail = file_name_tail (0, dir);
   invariant (!str_cmp_prefix (",,", dir_tail));
 
-  arch_finish_tree_commit (tree_root, archive, revision, changelog_loc);
+  arch_finish_tree_commit (tree_root, archive, revision, changelog_loc, full_meta);
 
   recycled_pristine_src = arch_find_pristine (0, tree_root, archive, prev_revision, arch_unlocked_pristine, arch_tree_pristine_search);
   if (recycled_pristine_src)
@@ -691,7 +737,7 @@ arch_finish_commit (int chatter_fd, t_uchar * tree_root, t_uchar * archive, t_uc
       inventory_path = file_name_in_vicinity (0, recycled_pristine_tmp, ",,index");
       rmrf_file (inventory_path);
 
-      arch_apply_changeset (&report, changeset, recycled_pristine_tmp, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, 0, 0, 0, 0);
+      arch_apply_changeset (&report, changeset, recycled_pristine_tmp, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, 0, 0, 0, 0, escape_classes);
       invariant (!arch_conflicts_occured (&report));
 
       arch_install_pristine (tree_root, archive, revision, recycled_pristine_tmp);
@@ -729,7 +775,7 @@ arch_commit_failed (t_uchar * tree_root, t_uchar * archive, t_uchar * revision,
 
 
 static void
-commit_make_changeset_callback (void * vfd, char * fmt, va_list ap)
+commit_make_changeset_callback (void * vfd, const char * fmt, va_list ap)
 {
   int fd;
 
