@@ -32,14 +32,23 @@
 
 
 /* __STDC__ prototypes for static functions */
+
+
+
 static t_uchar * arch_prepare_import_pristine (t_uchar ** changelog_loc_ret,
                                                t_uchar ** cooked_log_ret,
                                                t_uchar * tree_root,
                                                t_uchar * raw_log,
                                                t_uchar * archive,
-                                               t_uchar * version);
+                                               t_uchar * version,
+                                               int full_meta);
 static void arch_import_mid_commit (t_uchar * tree_root, t_uchar * cooked_log);
-static void arch_finish_import (t_uchar * tree_root, t_uchar * archive, t_uchar * version, t_uchar * pristine, t_uchar * changelog_loc);
+static void arch_finish_import (t_uchar * tree_root,
+                                t_uchar * archive,
+                                t_uchar * version,
+                                t_uchar * pristine,
+                                t_uchar * changelog_loc,
+                                int full_meta);
 static void arch_import_failed (t_uchar * tree_root, t_uchar * archive, t_uchar * version, t_uchar * pristine);
 static rel_table all_patches (t_uchar * tree_root);
 
@@ -58,10 +67,11 @@ arch_import (struct arch_archive * arch,
   t_uchar * my_uid = 0;
   t_uchar * txn_id = 0;
   t_uchar * revision;
+  const int full_meta = arch_tree_has_meta_flag (tree_root);
   
   revision = str_alloc_cat (0, version, "--base-0");
 
-  pristine_path = arch_prepare_import_pristine (&changelog_loc, &cooked_log, tree_root, raw_log, arch->name, version);
+  pristine_path = arch_prepare_import_pristine (&changelog_loc, &cooked_log, tree_root, raw_log, arch->name, version, full_meta);
 
   my_uid = arch_my_id_uid ();
   txn_id = arch_generate_txn_id ();
@@ -109,7 +119,7 @@ arch_import (struct arch_archive * arch,
       exit (2);
     }
 
-  arch_finish_import (tree_root, arch->name, version, pristine_path, changelog_loc);
+  arch_finish_import (tree_root, arch->name, version, pristine_path, changelog_loc, full_meta);
 
   arch_run_hook ("import", "ARCH_ARCHIVE", arch->name, "ARCH_REVISION", revision, "ARCH_TREE_ROOT", tree_root, 0);
   
@@ -130,7 +140,8 @@ arch_prepare_import_pristine (t_uchar ** changelog_loc_ret,
                               t_uchar * tree_root,
                               t_uchar * raw_log,
                               t_uchar * archive,
-                              t_uchar * version)
+                              t_uchar * version,
+                              int full_meta)
 {
   t_uchar * tmp_stem = 0;
   t_uchar * tmp_path = 0;
@@ -165,7 +176,7 @@ arch_prepare_import_pristine (t_uchar ** changelog_loc_ret,
 
   safe_mkdir (pristine_path, 0777);
 
-  copy_file_list (pristine_path, tree_root, inventory);
+  copy_file_list (pristine_path, tree_root, inventory, full_meta);
 
 
   /****************************************************************
@@ -308,9 +319,18 @@ arch_prepare_import_pristine (t_uchar ** changelog_loc_ret,
             changelog_path = file_name_in_vicinity (0, pristine_path, rel_peek_str (inventory, x, 0));
 
             safe_stat (changelog_path, &clstatb);
-            clmode = clstatb.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO);
+            if (full_meta)
+              {
+                clmode = clstatb.st_mode & 07777;
+              }
+            else
+              {
+                clmode = clstatb.st_mode & 0777;
+              }
             out_fd = safe_open (changelog_path, O_WRONLY | O_CREAT | O_TRUNC, clmode);
             safe_fchmod (out_fd, clmode);
+            if (full_meta)
+              safe_fchown (out_fd, clstatb.st_uid, clstatb.st_gid);
             arch_generate_changelog (out_fd, pristine_path, 0, 0, 0, 0, archive, version);
             safe_close (out_fd);
 
@@ -357,7 +377,12 @@ arch_import_mid_commit (t_uchar * tree_root, t_uchar * cooked_log)
 
 
 static void
-arch_finish_import (t_uchar * tree_root, t_uchar * archive, t_uchar * version, t_uchar * pristine, t_uchar * changelog_loc)
+arch_finish_import (t_uchar * tree_root,
+                    t_uchar * archive,
+                    t_uchar * version,
+                    t_uchar * pristine,
+                    t_uchar * changelog_loc,
+                    int full_meta)
 {
   t_uchar * revision = 0;
   t_uchar * pristine_dir = 0;
@@ -369,7 +394,7 @@ arch_finish_import (t_uchar * tree_root, t_uchar * archive, t_uchar * version, t
   /****************************************************************
    * txnally install the log file in the patch log
    */
-  arch_finish_tree_commit (tree_root, archive, revision, changelog_loc);
+  arch_finish_tree_commit (tree_root, archive, revision, changelog_loc, full_meta);
 
   /****************************************************************
    * Install the pristine tree unless we have a greedy library
