@@ -34,22 +34,22 @@ struct directory_regexps
 /* __STDC__ prototypes for static functions */
 static void copy_options_but_regexps (struct arch_inventory_options * dest,
                                       const struct arch_inventory_options * src);
-static void source_inventory_callback (t_uchar * path,
+static void source_inventory_callback (const t_uchar * path,
                                        struct stat * statb,
                                        enum arch_inventory_category category,
-                                       t_uchar * id,
+                                       const t_uchar * id,
                                        int has_source_name,
                                        void * closure,
                                        int escape_classes);
-static void source_inventory_files_callback (t_uchar * path,
+static void source_inventory_files_callback (const t_uchar * path,
                                              struct stat * statb,
                                              enum arch_inventory_category category,
-                                             t_uchar * id,
+                                             const t_uchar * id,
                                              int has_source_name,
                                              void * closure,
                                              int escape_classes);
 static void arch_inventory_traversal_internal (struct arch_inventory_options * options,
-                                               t_uchar * root,
+                                               const t_uchar * root,
                                                inv_callback callback,
                                                void * closure,
                                                assoc_table id_tagging_shortcut,
@@ -57,30 +57,39 @@ static void arch_inventory_traversal_internal (struct arch_inventory_options * o
                                                int escape_classes);
 static int cmp_files (const void * va, const void * vb);
 static int right_order_for_recursion (char * a, char * b);
-static int contains_illegal_character (char * filename);
-static int filename_matches (regex_t * pattern, char * filename);
-static int is_control_file (char * rel_file, char * filename);
-static int is_nested_tree (char * path);
-static int is_comment_line (t_uchar * line, long len);
-static int sets_re (char * kw, char ** re, t_uchar * line, long len);
-static int sets_id_tagging_method (char * kw,
+static int contains_illegal_character (const char * filename);
+static int filename_matches (regex_t * pattern, const char * filename);
+static int is_control_file (const char * rel_file, const char * filename);
+static int is_nested_tree (const char * path);
+static int is_comment_line (const t_uchar * line, long len);
+static int sets_re (const char * kw,
+                    char ** re,
+                    const t_uchar * line,
+                    long len);
+static int sets_id_tagging_method (const char * kw,
                                    enum arch_id_tagging_method * method_var,
                                    enum arch_inventory_category * untagged_category_var,
                                    enum arch_id_tagging_method method,
                                    enum arch_inventory_category untagged_category,
-                                   t_uchar * line, long len);
+                                   const t_uchar * line,
+                                   long len);
 static int sets_untagged_source_disposition (enum arch_inventory_category * untagged_category_var,
-                                             t_uchar * saved_line, long saved_len);
-static void read_directory_regexps (struct directory_regexps * regexps, char * dir_name);
+                                             const t_uchar * saved_line,
+                                             long saved_len);
+static void read_directory_regexps (struct directory_regexps * regexps,
+                                    const char * dir_name);
 static void free_directory_regexps (struct directory_regexps * regexps);
 
 
 
 rel_table
-arch_source_inventory (t_uchar * tree_root, int include_ctl, int include_precious, int include_nested)
+arch_source_inventory (const t_uchar * tree_root,
+                       int include_ctl,
+                       int include_precious,
+                       int include_nested)
 {
   int here_fd;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   struct arch_inventory_options options;
 
   here_fd = safe_open (".", O_RDONLY, 0);
@@ -104,11 +113,14 @@ arch_source_inventory (t_uchar * tree_root, int include_ctl, int include_preciou
   return answer;
 }
 
+
 rel_table
-arch_source_files_inventory (t_uchar * tree_root, int include_ctl, int include_precious)
+arch_source_files_inventory (const t_uchar * tree_root,
+                             int include_ctl,
+                             int include_precious)
 {
   int here_fd;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   struct arch_inventory_options options;
 
   here_fd = safe_open (".", O_RDONLY, 0);
@@ -148,26 +160,25 @@ copy_options_but_regexps (struct arch_inventory_options * dest,
 
 
 static void
-source_inventory_callback (t_uchar * path,
+source_inventory_callback (const t_uchar * path,
                            struct stat * statb,
                            enum arch_inventory_category category,
-                           t_uchar * id,
+                           const t_uchar * id,
                            int has_source_name,
                            void * closure,
                            int escape_classes)
 {
   rel_table * answer = (rel_table *)closure;
 
-  rel_add_records (answer, rel_make_record (path, id, 0), 0);
+  rel_add_records (answer, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), 0);
 }
 
 
-
 static void
-source_inventory_files_callback (t_uchar * path,
+source_inventory_files_callback (const t_uchar * path,
                                  struct stat * statb,
                                  enum arch_inventory_category category,
-                                 t_uchar * id,
+                                 const t_uchar * id,
                                  int has_source_name,
                                  void * closure,
                                  int escape_classes)
@@ -175,7 +186,7 @@ source_inventory_files_callback (t_uchar * path,
   rel_table * answer = (rel_table *)closure;
 
   if (!S_ISDIR (statb->st_mode))
-    rel_add_records (answer, rel_make_record (path, id, 0), 0);
+    rel_add_records (answer, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), 0);
 }
 
 
@@ -315,7 +326,7 @@ arch_null_default_naming_conventions_regexp (enum arch_inventory_category cat)
 
 void
 arch_get_inventory_naming_conventions (struct arch_inventory_options * options,
-                                       char * tree_root)
+                                       const char * tree_root)
 {
   char * excludes = 0;
   char * junk = 0;
@@ -449,7 +460,7 @@ arch_free_inventory_naming_conventions (struct arch_inventory_options * options)
 
 void
 arch_inventory_traversal (struct arch_inventory_options * options,
-                          t_uchar * root,
+                          const t_uchar * root,
                           inv_callback callback,
                           void * closure,
                           int escape_classes)
@@ -469,7 +480,7 @@ arch_inventory_traversal (struct arch_inventory_options * options,
 
 static void
 arch_inventory_traversal_internal (struct arch_inventory_options * options,
-                                   t_uchar * root,
+                                   const t_uchar * root,
                                    inv_callback callback,
                                    void * closure,
                                    assoc_table id_tagging_shortcut,
@@ -583,7 +594,16 @@ arch_inventory_traversal_internal (struct arch_inventory_options * options,
         }
 
       safe_lstat (rel_file, &stat_buf);
-
+      
+      /* only symlinks, directories and plain files are
+       * considerably source, all others need to be precious
+       */
+      if (   !S_ISREG (stat_buf.st_mode)
+          && !S_ISLNK (stat_buf.st_mode)
+          && !S_ISDIR (stat_buf.st_mode))
+        {
+          goto precious_file;
+        }
 
       /* non-printing characters, spaces, and glob characters are
        * mandatory unrecognized files
@@ -879,6 +899,7 @@ cmp_files (const void * va, const void * vb)
   return str_cmp (a, b);
 }
 
+
 static int
 right_order_for_recursion (char * a, char * b)
 {
@@ -911,7 +932,7 @@ right_order_for_recursion (char * a, char * b)
 
 
 static int
-contains_illegal_character (char * filename)
+contains_illegal_character (const char * filename)
 {
   int x;
 
@@ -926,8 +947,9 @@ contains_illegal_character (char * filename)
   return 0;
 }
 
+
 static int
-filename_matches (regex_t * pattern, char * filename)
+filename_matches (regex_t * pattern, const char * filename)
 {
   int answer;
 
@@ -943,8 +965,9 @@ filename_matches (regex_t * pattern, char * filename)
   return -1;
 }
 
+
 static int
-is_control_file (char * rel_file, char * filename)
+is_control_file (const char * rel_file, const char * filename)
 {
   static int compiled = 0;
   static regex_t control_pattern = {0,};
@@ -965,7 +988,7 @@ is_control_file (char * rel_file, char * filename)
 
 
 static int
-is_nested_tree (char * path)
+is_nested_tree (const char * path)
 {
   int errn;
   t_uchar * ctl_file;
@@ -991,14 +1014,19 @@ is_nested_tree (char * path)
   return 1;
 }
 
+
 static int
-is_comment_line (t_uchar * line, long len)
+is_comment_line (const t_uchar * line, long len)
 {
   return !len || char_is_space (line[0]) || (line[0] == '#');
 }
 
+
 static int
-sets_re (char * kw, char ** re, t_uchar * line, long len)
+sets_re (const char * kw,
+         char ** re,
+         const t_uchar * line,
+         long len)
 {
   int l;
   t_uchar * tem;
@@ -1043,13 +1071,15 @@ sets_re (char * kw, char ** re, t_uchar * line, long len)
   return 1;
 }
 
+
 static int
-sets_id_tagging_method (char * kw,
-                     enum arch_id_tagging_method * method_var,
-                     enum arch_inventory_category * untagged_category_var,
-                     enum arch_id_tagging_method method,
-                     enum arch_inventory_category untagged_category,
-                     t_uchar * line, long len)
+sets_id_tagging_method (const char * kw,
+                        enum arch_id_tagging_method * method_var,
+                        enum arch_inventory_category * untagged_category_var,
+                        enum arch_id_tagging_method method,
+                        enum arch_inventory_category untagged_category,
+                        const t_uchar * line,
+                        long len)
 {
   int l;
 
@@ -1066,11 +1096,13 @@ sets_id_tagging_method (char * kw,
   return 1;
 }
 
+
 static int
 sets_untagged_source_disposition (enum arch_inventory_category * untagged_category_var,
-                                  t_uchar * saved_line, long saved_len)
+                                  const t_uchar * saved_line,
+                                  long saved_len)
 {
-  t_uchar * line = saved_line;
+  const t_uchar * line = saved_line;
   long len = saved_len;
   t_uchar * kw = "untagged-source";
   int l;
@@ -1094,15 +1126,19 @@ sets_untagged_source_disposition (enum arch_inventory_category * untagged_catego
       --len;
     }
 
-  spec = line;
+  {
+    const t_uchar * tmp;
 
-  while (len && char_is_alpha (line[0]))
-    {
-      ++line;
-      --len;
-    }
+    tmp = line;
 
-  spec = str_save_n (0, spec, line - spec);
+    while (len && char_is_alpha (line[0]))
+      {
+        ++line;
+        --len;
+      }
+
+    spec = str_save_n (0, tmp, line - tmp);
+  }
 
   while (len && char_is_blank (line[0]))
     {
@@ -1148,8 +1184,10 @@ sets_untagged_source_disposition (enum arch_inventory_category * untagged_catego
   return 1;
 }
 
+
 static void
-read_directory_regexps (struct directory_regexps * regexps, char * dir_name)
+read_directory_regexps (struct directory_regexps * regexps,
+                        const char * dir_name)
 {
   char * excludes = 0;
   char * junk = 0;
@@ -1251,6 +1289,7 @@ read_directory_regexps (struct directory_regexps * regexps, char * dir_name)
   lim_free (0, source);
 }
 
+
 static void
 free_directory_regexps (struct directory_regexps * regexps)
 {
