@@ -20,9 +20,14 @@
 
 
 /* __STDC__ prototypes for static functions */
-static int find_id_in_inventory (rel_table invent, t_uchar * id);
-static void double_check_file_identity (t_uchar * from_here_path, t_uchar * tree_root, t_uchar * loc);
-
+static void __attribute__((format (printf, 2, 3))) invoke_report_callback (struct arch_make_changeset_report * report,
+                                                                           const char * fmt, ...);
+     static int find_id_in_inventory (rel_table invent,
+                                      const t_uchar * id);
+     static void double_check_file_identity (const t_uchar * from_here_path,
+                                             const t_uchar * tree_root,
+                                             const t_uchar * loc);
+     
 
 #if !defined(__GNUC__)
 #  undef __attribute__
@@ -30,7 +35,8 @@ static void double_check_file_identity (t_uchar * from_here_path, t_uchar * tree
 #endif
 
 static void __attribute__((format (printf, 2, 3)))
-invoke_report_callback (struct arch_make_changeset_report * report, char * fmt, ...)
+invoke_report_callback (struct arch_make_changeset_report * report,
+                        const char * fmt, ...)
 {
   va_list ap;
 
@@ -45,16 +51,17 @@ invoke_report_callback (struct arch_make_changeset_report * report, char * fmt,
 
 void
 arch_make_files_changeset (struct arch_make_changeset_report * report,
-                           t_uchar * dest,
+                           const t_uchar * dest,
                            rel_table file_list,
-                           t_uchar * orig, t_uchar * mod,
+                           const t_uchar * orig,
+                           const t_uchar * mod,
                            enum arch_id_tagging_method method,
                            enum arch_inventory_category untagged_source_category,
                            int escape_classes)
 {
   int x;
 
-  struct arch_changeset_report csr = {0,};
+  struct arch_changeset_report csr = {rel_table_nil,};
 
   if (method == arch_unspecified_id_tagging)
     {
@@ -64,8 +71,7 @@ arch_make_files_changeset (struct arch_make_changeset_report * report,
   mem_set0 ((t_uchar *)&report->orig_index, sizeof (report->orig_index));
   mem_set0 ((t_uchar *)&report->mod_index, sizeof (report->mod_index));
 
-  arch_changeset_inventory (&report->orig_index, orig, orig, method,
-                            untagged_source_category, escape_classes);
+  arch_changeset_inventory (&report->orig_index, orig, orig, method, untagged_source_category, escape_classes);
   report->mod_index.dirs = rel_copy_table (report->orig_index.dirs);
   report->mod_index.files = rel_copy_table (report->orig_index.files);
 
@@ -85,10 +91,10 @@ arch_make_files_changeset (struct arch_make_changeset_report * report,
   report->mod_file_id_of = rel_to_assoc (report->mod_index.files, 0, 1);
   report->mod_file_loc_of = rel_to_assoc (report->mod_index.files, 1, 0);
 
-  assoc_set (&report->orig_dir_id_of, ".", "?_.");
-  assoc_set (&report->mod_dir_id_of, ".", "?_.");
-  assoc_set (&report->orig_dir_loc_of, "?_.", ".");
-  assoc_set (&report->mod_dir_loc_of, "?_.", ".");
+  assoc_set_taking (&report->orig_dir_id_of, rel_make_field_str ("."), rel_make_field_str ("?_."));
+  assoc_set_taking (&report->mod_dir_id_of, rel_make_field_str ("."), rel_make_field_str ("?_."));
+  assoc_set_taking (&report->orig_dir_loc_of, rel_make_field_str ("?_."), rel_make_field_str ("."));
+  assoc_set_taking (&report->mod_dir_loc_of, rel_make_field_str ("?_."), rel_make_field_str ("."));
 
   arch_make_changeset_compute_container_map (&report->orig_container_dir_id_of_dir_id, report->orig_dir_id_of, report->orig_index.dirs);
   arch_make_changeset_compute_container_map (&report->orig_container_dir_id_of_file_id, report->orig_dir_id_of, report->orig_index.files);
@@ -106,37 +112,37 @@ arch_make_files_changeset (struct arch_make_changeset_report * report,
       t_uchar * mod_path = 0;
       int diff_fd;
 
-      id = arch_inventory_id (method, untagged_source_category, file_list[x][0], 0, 0, 0);
+      id = arch_inventory_id (method, untagged_source_category, rel_peek_str (file_list, x, 0), 0, 0, 0);
 
       orig_index = find_id_in_inventory (report->orig_index.files, id);
       mod_index = find_id_in_inventory (report->mod_index.files, id);
 
       if (orig_index < 0)
         {
-          safe_printfmt (2, "make-changeset-files: file missing from ORIG tree (%s)\n", file_list[x][0]);
+          safe_printfmt (2, "make-changeset-files: file missing from ORIG tree (%s)\n", rel_peek_str (file_list, x, 0));
           exit (1);
         }
 
       if (mod_index < 0)
         {
-          safe_printfmt (2, "make-changeset-files: file missing from MOD tree (%s)\n", file_list[x][0]);
+          safe_printfmt (2, "make-changeset-files: file missing from MOD tree (%s)\n", rel_peek_str (file_list, x, 0));
           exit (1);
         }
 
-      double_check_file_identity (file_list[x][0], mod, report->mod_index.files[mod_index][0]);
+      double_check_file_identity (rel_peek_str (file_list, x, 0), mod, rel_peek_str (report->mod_index.files, mod_index, 0));
 
-      orig_path = file_name_in_vicinity (0, orig, report->orig_index.files[orig_index][0]);
-      mod_path = file_name_in_vicinity (0, mod, report->mod_index.files[mod_index][0]);
+      orig_path = file_name_in_vicinity (0, orig, rel_peek_str (report->orig_index.files, orig_index, 0));
+      mod_path = file_name_in_vicinity (0, mod, rel_peek_str (report->mod_index.files, mod_index, 0));
 
       if (arch_binary_files_differ (orig_path, mod_path, 0, 0))
         {
           int diff_stat; 
           t_uchar * escaped_tmp;
 
-          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, file_list[x][0]);
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, rel_peek_str (file_list, x, 0));
           invoke_report_callback (report, "M  %s\n", escaped_tmp);
-          diff_fd = arch_changeset_add_diffs (&csr, report, dest, report->orig_index.files[orig_index][0], report->mod_index.files[mod_index][0], id);
-          diff_stat = arch_invoke_diff (diff_fd, orig_path, report->orig_index.files[orig_index][0], mod_path, report->orig_index.files[orig_index][0], 0, 0);
+          diff_fd = arch_changeset_add_diffs (&csr, report, dest, rel_peek_str (report->orig_index.files, orig_index, 0), rel_peek_str (report->mod_index.files, mod_index, 0), id);
+          diff_stat = arch_invoke_diff (diff_fd, orig_path, rel_peek_str (report->orig_index.files, orig_index, 0), mod_path, rel_peek_str (report->orig_index.files, orig_index, 0), 0, 0);
           safe_close (diff_fd);
 
           lim_free (0, escaped_tmp);
@@ -150,7 +156,7 @@ arch_make_files_changeset (struct arch_make_changeset_report * report,
               long amt;
 
               patches_path = file_name_in_vicinity (0, dest, "patches");
-              patch_path = file_name_in_vicinity (0, patches_path, report->mod_index.files[mod_index][0]);
+              patch_path = file_name_in_vicinity (0, patches_path, rel_peek_str (report->mod_index.files, mod_index, 0));
               patch_path = str_realloc_cat (0, patch_path, ".patch");
 
               diff_in_fd = safe_open (patch_path, O_RDONLY, 0);
@@ -192,13 +198,14 @@ arch_make_files_changeset (struct arch_make_changeset_report * report,
 
 
 static int
-find_id_in_inventory (rel_table invent, t_uchar * id)
+find_id_in_inventory (rel_table invent,
+                      const t_uchar * id)
 {
   int x;
 
   for (x = 0; x < rel_n_records (invent); ++x)
     {
-      if (!str_cmp (id, invent[x][1]))
+      if (!str_cmp (id, rel_peek_str (invent, x, 1)))
         return x;
     }
 
@@ -207,7 +214,9 @@ find_id_in_inventory (rel_table invent, t_uchar * id)
 
 
 static void
-double_check_file_identity (t_uchar * from_here_path, t_uchar * tree_root, t_uchar * loc)
+double_check_file_identity (const t_uchar * from_here_path,
+                            const t_uchar * tree_root,
+                            const t_uchar * loc)
 {
   struct stat users_stat;
   t_uchar * computed_path = 0;
