@@ -20,21 +20,19 @@
 
 
 /* __STDC__ prototypes for static functions */
-static int arch_cmp_by_field (void * va, void * vb, void * vdata);
-static int arch_lvl_cmp_by_field (void * va, void * vb, void * vdata);
-static t_uchar * over_opt_archive_prefix (t_uchar * name);
-static t_uchar * over_archive_name (t_uchar * in);
-static t_uchar * over_basename (t_uchar * in);
-static t_uchar * over_separator (t_uchar * in);
-static t_uchar * over_version (t_uchar * in);
-static t_uchar * over_patch_level (t_uchar * in);
+static const t_uchar * over_opt_archive_prefix (const t_uchar * name);
+static const t_uchar * over_archive_name (const t_uchar * in);
+static const t_uchar * over_basename (const t_uchar * in);
+static const t_uchar * over_separator (const t_uchar * in);
+static const t_uchar * over_version (const t_uchar * in);
+static const t_uchar * over_patch_level (const t_uchar * in);
 
 
 
 int
-arch_valid_id (t_uchar * id)
+arch_valid_id (const t_uchar * id)
 {
-  t_uchar * non_empty_marker;
+  const t_uchar * non_empty_marker;
 
   /* This isn't exactly the same as larch's version --
    * it's a superset.   Both are wrong, really. :-)
@@ -87,9 +85,9 @@ arch_valid_id (t_uchar * id)
 }
 
 int
-arch_valid_archive_name (t_uchar * name)
+arch_valid_archive_name (const t_uchar * name)
 {
-  t_uchar * end;
+  const t_uchar * end;
 
   if (!name)
     return 0;
@@ -103,7 +101,7 @@ arch_valid_archive_name (t_uchar * name)
 }
 
 int
-arch_valid_patch_level_name (t_uchar * name)
+arch_valid_patch_level_name (const t_uchar * name)
 {
   t_uchar * lvl;
 
@@ -132,14 +130,14 @@ arch_valid_patch_level_name (t_uchar * name)
 }
 
 int
-arch_valid_config_name (t_uchar * name)
+arch_valid_config_name (const t_uchar * name)
 {
   return is_non_upwards_relative_path (name);
 }
 
 
 /*(c arch_valid_package_name)
- * int arch_valid_package_name (t_uchar * name,
+ * int arch_valid_package_name (const t_uchar * name,
  *                              enum arch_valid_package_name_archive archive_disposition,
  *                              enum arch_valid_package_name_types type,
  *                              int tolerant);
@@ -181,7 +179,7 @@ arch_valid_config_name (t_uchar * name)
  *         if 1, then the name may have additional components
  */
 int
-arch_valid_package_name (t_uchar * name,
+arch_valid_package_name (const t_uchar * name,
                          enum arch_valid_package_name_archive archive_disposition,
                          enum arch_valid_package_name_types type,
                          int tolerant)
@@ -191,7 +189,7 @@ arch_valid_package_name (t_uchar * name,
   int has_branch;
   int has_version;
   int has_patch_level;
-  t_uchar * next;
+  const t_uchar * next;
 
   if (!name)
     return 0;
@@ -325,12 +323,12 @@ arch_valid_package_name (t_uchar * name,
 
 
 /*(c arch_is_system_package_name)
- * int arch_is_system_package_name (t_uchar * name);
+ * int arch_is_system_package_name (const t_uchar * name);
  * 
  * Return non-0 if `name' is a system package name.
  */
 int
-arch_is_system_package_name (t_uchar * name)
+arch_is_system_package_name (const t_uchar * name)
 {
   t_uchar * package = 0;
   int answer;
@@ -344,8 +342,8 @@ arch_is_system_package_name (t_uchar * name)
 
 /*(c arch_parse_package_name)
  * t_uchar * arch_parse_package_name (enum arch_parse_package_name_type type,
- *                                    t_uchar * default_archive,
- *                                    t_uchar * name);
+ *                                    const t_uchar * default_archive,
+ *                                    const t_uchar * name);
  *
  * Parse a package name.
  *
@@ -385,8 +383,8 @@ arch_is_system_package_name (t_uchar * name)
  */
 t_uchar *
 arch_parse_package_name (enum arch_parse_package_name_type type,
-                         t_uchar * default_archive,
-                         t_uchar * name)
+                         const t_uchar * default_archive,
+                         const t_uchar * name)
 {
   invariant (arch_valid_package_name (name, arch_maybe_archive, arch_req_package, 1));
 
@@ -435,7 +433,7 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 
     case arch_ret_category:
       {
-        t_uchar * cat_end;
+        const t_uchar * cat_end;
 
         name = over_opt_archive_prefix (name);
         cat_end = over_basename (name);
@@ -445,7 +443,7 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 
     case arch_ret_branch:
       {
-        t_uchar * branch_end;
+        const t_uchar * branch_end;
 
         name = over_opt_archive_prefix (name);
         name = over_basename (name); /* over category */
@@ -461,8 +459,8 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 
     case arch_ret_package:
       {
-        t_uchar * branch_end;
-        t_uchar * t;
+        const t_uchar * branch_end;
+        const t_uchar * t;
 
         name = over_opt_archive_prefix (name);
         branch_end = over_basename (name);
@@ -479,9 +477,9 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 
     case arch_ret_version:
       {
-        t_uchar * version_start;
-        t_uchar * version_end;
-        t_uchar * t;
+        const t_uchar * version_start;
+        const t_uchar * version_end;
+        const t_uchar * t;
 
         name = over_opt_archive_prefix (name);
         version_end = over_basename (name); /* over category */
@@ -503,7 +501,7 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 
     case arch_ret_patch_level:
       {
-        t_uchar * t;
+        const t_uchar * t;
 
         name = over_opt_archive_prefix (name);
         invariant (!!name);
@@ -529,9 +527,9 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 
     case arch_ret_package_version:
       {
-        t_uchar * version_start;
-        t_uchar * version_end;
-        t_uchar * t;
+        const t_uchar * version_start;
+        const t_uchar * version_end;
+        const t_uchar * t;
 
         name = over_opt_archive_prefix (name);
         version_end = over_basename (name); /* over category */
@@ -554,7 +552,8 @@ arch_parse_package_name (enum arch_parse_package_name_type type,
 }
 
 t_uchar *
-arch_fully_qualify (t_uchar * default_archive, t_uchar * name)
+arch_fully_qualify (const t_uchar * default_archive,
+                    const t_uchar * name)
 {
   invariant (arch_valid_package_name (name, arch_maybe_archive, arch_req_package, 1));
   if (default_archive)
@@ -575,10 +574,11 @@ arch_fully_qualify (t_uchar * default_archive, t_uchar * name)
 }
 
 int
-arch_names_cmp (t_uchar * a, t_uchar * b)
+arch_names_cmp (const t_uchar * a,
+                const t_uchar * b)
 {
-  t_uchar * a_nxt;
-  t_uchar * b_nxt;
+  const t_uchar * a_nxt;
+  const t_uchar * b_nxt;
   int res;
 
   /* compare archive name
@@ -684,8 +684,8 @@ arch_names_cmp (t_uchar * a, t_uchar * b)
     while (1)
       {
         int errn;
-        t_uchar * a_comp_end;
-        t_uchar * b_comp_end;
+        const t_uchar * a_comp_end;
+        const t_uchar * b_comp_end;
         t_ulong a_comp;
         t_ulong b_comp;
 
@@ -797,46 +797,21 @@ arch_names_cmp (t_uchar * a, t_uchar * b)
   }
 }
 
-struct arch_sort_spec
-{
-  int reverse_p;
-  int field;
-};
-
 void
 arch_sort_table_by_name_field (int reverse_p, rel_table table, int field)
 {
-  struct arch_sort_spec spec;
-
-  spec.reverse_p = reverse_p;
-  spec.field = field;
-
-  quicksort ((void *)table, rel_n_records (table), sizeof (rel_record), arch_cmp_by_field, (void *)&spec);
+  rel_sort_table_by_field_fn (reverse_p, table, field, arch_names_cmp);
 }
 
-static int
-arch_cmp_by_field (void * va, void * vb, void * vdata)
+void
+arch_sort_table_by_patch_level_field (int reverse_p, rel_table table, int field)
 {
-  rel_record * a;
-  rel_record * b;
-  struct arch_sort_spec * spec;
-
-  a = (rel_record *)va;
-  b = (rel_record *)vb;
-  spec = (struct arch_sort_spec *)vdata;
-
-  if (spec->reverse_p)
-    {
-      return -arch_names_cmp ((*a)[spec->field], (*b)[spec->field]);
-    }
-  else
-    {
-      return arch_names_cmp ((*a)[spec->field], (*b)[spec->field]);
-    }
+  rel_sort_table_by_field_fn (reverse_p, table, field, arch_patch_lvl_cmp);
 }
 
 int
-arch_patch_lvl_cmp (t_uchar * a, t_uchar * b)
+arch_patch_lvl_cmp (const t_uchar * a,
+                    const t_uchar * b)
 {
   t_uchar * a_dash;
   t_uchar * b_dash;
@@ -864,49 +839,17 @@ arch_patch_lvl_cmp (t_uchar * a, t_uchar * b)
     return 0;
 }
 
-void
-arch_sort_table_by_patch_level_field (int reverse_p, rel_table table, int field)
-{
-  struct arch_sort_spec spec;
-
-  spec.reverse_p = reverse_p;
-  spec.field = field;
-
-  quicksort ((void *)table, rel_n_records (table), sizeof (rel_record), arch_lvl_cmp_by_field, (void *)&spec);
-}
-
-static int
-arch_lvl_cmp_by_field (void * va, void * vb, void * vdata)
-{
-  rel_record * a;
-  rel_record * b;
-  struct arch_sort_spec * spec;
-
-  a = (rel_record *)va;
-  b = (rel_record *)vb;
-  spec = (struct arch_sort_spec *)vdata;
-
-  if (spec->reverse_p)
-    {
-      return -arch_patch_lvl_cmp ((*a)[spec->field], (*b)[spec->field]);
-    }
-  else
-    {
-      return arch_patch_lvl_cmp ((*a)[spec->field], (*b)[spec->field]);
-    }
-}
-
 
 rel_table
 arch_pick_archives_by_field (rel_table in, int field)
 {
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (in); ++x)
     {
-      if (arch_valid_archive_name (in[x][field]))
-        rel_add_records (&answer, rel_copy_record (in[x]), 0);
+      if (arch_valid_archive_name (rel_peek_str (in, x, field)))
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (in, x)), 0);
     }
 
   return answer;
@@ -916,13 +859,13 @@ arch_pick_archives_by_field (rel_table in, int field)
 rel_table
 arch_pick_categories_by_field (rel_table in, int field)
 {
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (in); ++x)
     {
-      if (arch_valid_package_name (in[x][field], arch_no_archive, arch_req_category, 0))
-        rel_add_records (&answer, rel_copy_record (in[x]), 0);
+      if (arch_valid_package_name (rel_peek_str (in, x, field), arch_no_archive, arch_req_category, 0))
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (in, x)), 0);
     }
 
   return answer;
@@ -931,13 +874,13 @@ arch_pick_categories_by_field (rel_table in, int field)
 rel_table
 arch_pick_branches_by_field (rel_table in, int field)
 {
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (in); ++x)
     {
-      if (arch_valid_package_name (in[x][field], arch_no_archive, arch_req_package, 0))
-        rel_add_records (&answer, rel_copy_record (in[x]), 0);
+      if (arch_valid_package_name (rel_peek_str (in, x, field), arch_no_archive, arch_req_package, 0))
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (in, x)), 0);
     }
 
   return answer;
@@ -946,13 +889,13 @@ arch_pick_branches_by_field (rel_table in, int field)
 rel_table
 arch_pick_versions_by_field (rel_table in, int field)
 {
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (in); ++x)
     {
-      if (arch_valid_package_name (in[x][field], arch_no_archive, arch_req_version, 0))
-        rel_add_records (&answer, rel_copy_record (in[x]), 0);
+      if (arch_valid_package_name (rel_peek_str (in, x, field), arch_no_archive, arch_req_version, 0))
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (in, x)), 0);
     }
 
   return answer;
@@ -961,13 +904,13 @@ arch_pick_versions_by_field (rel_table in, int field)
 rel_table
 arch_pick_revisions_by_field (rel_table in, int field)
 {
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (in); ++x)
     {
-      if (arch_valid_package_name (in[x][field], arch_no_archive, arch_req_patch_level, 0))
-        rel_add_records (&answer, rel_copy_record (in[x]), 0);
+      if (arch_valid_package_name (rel_peek_str (in, x, field), arch_no_archive, arch_req_patch_level, 0))
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (in, x)), 0);
     }
 
   return answer;
@@ -976,13 +919,13 @@ arch_pick_revisions_by_field (rel_table in, int field)
 rel_table
 arch_pick_patch_levels_by_field (rel_table in, int field)
 {
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (in); ++x)
     {
-      if (arch_valid_patch_level_name (in[x][field]))
-        rel_add_records (&answer, rel_copy_record (in[x]), 0);
+      if (arch_valid_patch_level_name (rel_peek_str (in, x, field)))
+        rel_add_records (&answer, rel_copy_record (rel_peek_record (in, x)), 0);
     }
 
   return answer;
@@ -990,11 +933,12 @@ arch_pick_patch_levels_by_field (rel_table in, int field)
 
 
 enum arch_patch_level_type
-arch_analyze_patch_level (t_ulong * n, t_uchar * patch_level)
+arch_analyze_patch_level (t_ulong * n,
+                          const t_uchar * patch_level)
 {
   int ign;
   enum arch_patch_level_type type;
-  t_uchar * n_str;
+  const t_uchar * n_str;
 
   if (!str_cmp ("base-0", patch_level))
     {
@@ -1061,13 +1005,13 @@ arch_form_patch_level (enum arch_patch_level_type type, t_ulong n)
 
 
 
-static t_uchar *
-over_opt_archive_prefix (t_uchar * name)
+static const t_uchar *
+over_opt_archive_prefix (const t_uchar * name)
 {
-  t_uchar * slash;
-  t_uchar * archive_name_end;
+  const t_uchar * slash;
+  const t_uchar * archive_name_end;
 
-  slash = str_chr_index (name, '/');
+  slash = (const t_uchar *)str_chr_index (name, '/');
 
   if (!slash)
     return name;
@@ -1083,8 +1027,8 @@ over_opt_archive_prefix (t_uchar * name)
 
 #define char_is_alnum_or_dash(c) (char_is_alnum (c) || (c == '-'))
 
-static t_uchar *
-over_archive_name (t_uchar * in)
+static const t_uchar *
+over_archive_name (const t_uchar * in)
 {
   if (!char_is_alnum_or_dash (*in))
     return 0;
@@ -1133,8 +1077,8 @@ over_archive_name (t_uchar * in)
 }
 
 
-static t_uchar *
-over_basename (t_uchar * in)
+static const t_uchar *
+over_basename (const t_uchar * in)
 {
   if (!char_is_alpha (*in))
     return 0;
@@ -1159,8 +1103,8 @@ over_basename (t_uchar * in)
   return 0;
 }
 
-static t_uchar *
-over_separator (t_uchar * in)
+static const t_uchar *
+over_separator (const t_uchar * in)
 {
   if ((*in == '-') && (in[1] == '-'))
     return in + 2;
@@ -1168,8 +1112,8 @@ over_separator (t_uchar * in)
     return 0;
 }
 
-static t_uchar *
-over_version (t_uchar * in)
+static const t_uchar *
+over_version (const t_uchar * in)
 {
   while (1)
     {
@@ -1194,8 +1138,8 @@ over_version (t_uchar * in)
   return 0;
 }
 
-static t_uchar *
-over_patch_level (t_uchar * in)
+static const t_uchar *
+over_patch_level (const t_uchar * in)
 {
   if (!str_cmp (in, "base-0"))
     return in + sizeof ("base-0") - 1;
