@@ -109,25 +109,67 @@ struct sftp_attrs
 
 
 /* __STDC__ prototypes for static functions */
-static t_uchar * pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static rel_table pfs_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_file_exists (struct arch_pfs_session * p, t_uchar * path);
-static int pfs_get_file (struct arch_pfs_session * p, int out_fd, t_uchar * path, int soft_errors);
-static int pfs_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t mode, int soft_errors);
-static int pfs_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from, t_uchar * to, int soft_errors);
-static int pfs_is_dir (struct arch_pfs_session * p, t_uchar * path);
-static int pfs_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int in_fd, int soft_errors);
-
-static t_uchar * sftp_abs_path (t_uchar * cwd, t_uchar * path);
-static void sftp_client_do_init (struct arch_pfs_sftp_session * p, int soft_errors);
-static int sftp_write_pkt (struct arch_pfs_sftp_session * p, int soft_errors, char *fmt, ...);
-static int sftp_server_response (struct arch_pfs_sftp_session * p, char **pkt, int *pkt_len, int expected_response, int soft_errors);
-static int sftp_decode_status (char *pkt, int pkt_len, int i, int soft_errors);
-static int sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path, int soft_errors);
-static int sftp_import_attrs (struct sftp_attrs *a, char *pkt, unsigned int pkt_len, int soft_errors);
-static int sftp_export_attrs (struct sftp_attrs *a, char **pkt, int *len);
+static int sftp_write_pkt (struct arch_pfs_sftp_session * p,
+                           int soft_errors,
+                           const char *fmt, ...);
+static int sftp_export_attrs (struct sftp_attrs *a,
+                              char ** pkt,
+                              int * len);
+static void sftp_client_do_init (struct arch_pfs_sftp_session * p,
+                                 int soft_errors);
+static int sftp_server_response (struct arch_pfs_sftp_session * p,
+                                 char ** pkt,
+                                 int * pkt_len,
+                                 int expected_response,
+                                 int soft_errors);
+static int sftp_decode_status (char * pkt,
+                               int pkt_len,
+                               int i,
+                               int soft_errors);
+static t_uchar * sftp_abs_path (const t_uchar * cwd,
+                                const t_uchar *path);
+static int sftp_get (struct arch_pfs_sftp_session * p,
+                     int data_fd,
+                     const t_uchar * path,
+                     int soft_errors);
+static int pfs_get_file (struct arch_pfs_session * p,
+                         int out_fd,
+                         const t_uchar * path,
+                         int soft_errors);
+static t_uchar * pfs_file_contents (struct arch_pfs_session * p,
+                                    const t_uchar * path,
+                                    int soft_errors);
+static rel_table pfs_directory_files (struct arch_pfs_session * p,
+                                      const t_uchar * path,
+                                      int soft_errors);
+static int sftp_import_attrs (struct sftp_attrs *a,
+                              char * pkt,
+                              unsigned int pkt_len,
+                              int soft_errors);
+static int pfs_put_file (struct arch_pfs_session *p,
+                         const t_uchar * path,
+                         mode_t perms,
+                         int data_fd,
+                         int soft_errors);
+static int pfs_mkdir (struct arch_pfs_session * p,
+                      const t_uchar * path,
+                      mode_t mode,
+                      int soft_errors);
+static int pfs_file_exists (struct arch_pfs_session * p,
+                            const t_uchar * path);
+static int pfs_is_dir (struct arch_pfs_session * p,
+                       const t_uchar * path);
+static int pfs_rename (struct arch_pfs_session * p,
+                       t_uchar ** errstr,
+                       const t_uchar * from,
+                       const t_uchar * to,
+                       int soft_errors);
+static int pfs_rmdir (struct arch_pfs_session * p,
+                      const t_uchar * path,
+                      int soft_errors);
+static int pfs_rm (struct arch_pfs_session * p,
+                   const t_uchar * path,
+                   int soft_errors);
 static t_uchar * dirfold (t_uchar *dir);
 
 
@@ -154,7 +196,7 @@ struct arch_pfs_vtable sftp_pfs_fns =
 
 
 int
-arch_pfs_sftp_supported_protocol (t_uchar * uri)
+arch_pfs_sftp_supported_protocol (const t_uchar * uri)
 {
   if (!str_cmp_prefix ("sftp:", uri))
     return 1;
@@ -164,7 +206,9 @@ arch_pfs_sftp_supported_protocol (t_uchar * uri)
 
 
 static int
-sftp_write_pkt (struct arch_pfs_sftp_session * p, int soft_errors, char *fmt, ...)
+sftp_write_pkt (struct arch_pfs_sftp_session * p,
+                int soft_errors,
+                const char *fmt, ...)
 {
   va_list ap;
   struct sftp_attrs *attrs;
@@ -254,8 +298,10 @@ sftp_write_pkt (struct arch_pfs_sftp_session * p, int soft_errors, char *fmt, ..
 
 
 /* take the structure and create an sftp attr string, returns size of string */
-int
-sftp_export_attrs (struct sftp_attrs *a, char **pkt, int *len)
+static int
+sftp_export_attrs (struct sftp_attrs *a,
+                   char ** pkt,
+                   int * len)
 {
   int i = 0;
 
@@ -306,7 +352,7 @@ sftp_export_attrs (struct sftp_attrs *a, char **pkt, int *len)
 
 
 struct arch_pfs_session *
-arch_pfs_sftp_connect (t_uchar * uri)
+arch_pfs_sftp_connect (const t_uchar * uri)
 {
   struct arch_pfs_sftp_session * answer = 0;
   t_uchar * hostname = 0;
@@ -416,7 +462,8 @@ arch_pfs_sftp_connect (t_uchar * uri)
 
 
 static void
-sftp_client_do_init (struct arch_pfs_sftp_session * p, int soft_errors)
+sftp_client_do_init (struct arch_pfs_sftp_session * p,
+                     int soft_errors)
 {
   int pkt_len;
   char *pkt;
@@ -431,8 +478,11 @@ sftp_client_do_init (struct arch_pfs_sftp_session * p, int soft_errors)
 
 
 static int
-sftp_server_response (struct arch_pfs_sftp_session * p, char **pkt,
-                      int *pkt_len, int expected_response, int soft_errors)
+sftp_server_response (struct arch_pfs_sftp_session * p,
+                      char ** pkt,
+                      int * pkt_len,
+                      int expected_response,
+                      int soft_errors)
 {
   int i, errn;
   unsigned int len, read_len, total_read, response;
@@ -530,7 +580,10 @@ sftp_server_response (struct arch_pfs_sftp_session * p, char **pkt,
 
 
 static int
-sftp_decode_status (char *pkt, int pkt_len, int i, int soft_errors)
+sftp_decode_status (char * pkt,
+                    int pkt_len,
+                    int i,
+                    int soft_errors)
 {
   int code, err_len;
 
@@ -567,7 +620,8 @@ sftp_decode_status (char *pkt, int pkt_len, int i, int soft_errors)
 
 
 static t_uchar *
-sftp_abs_path (t_uchar * cwd, t_uchar *path)
+sftp_abs_path (const t_uchar * cwd,
+               const t_uchar *path)
 {
   t_uchar * ap;
   if (path[0] != '/')
@@ -581,7 +635,9 @@ sftp_abs_path (t_uchar * cwd, t_uchar *path)
 
 
 static int
-sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
+sftp_get (struct arch_pfs_sftp_session * p,
+          int data_fd,
+          const t_uchar * path,
           int soft_errors)
 {
   struct sftp_attrs attrs;
@@ -728,7 +784,9 @@ sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
 
 
 static int
-pfs_get_file (struct arch_pfs_session * p, int out_fd, t_uchar * path,
+pfs_get_file (struct arch_pfs_session * p,
+              int out_fd,
+              const t_uchar * path,
               int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
@@ -738,7 +796,9 @@ pfs_get_file (struct arch_pfs_session * p, int out_fd, t_uchar * path,
 
 
 static t_uchar *
-pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_file_contents (struct arch_pfs_session * p,
+                   const t_uchar * path,
+                   int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   t_uchar * tmp_path = 0;
@@ -764,12 +824,13 @@ pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
 
 
 static rel_table
-pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
+pfs_directory_files (struct arch_pfs_session * p,
+                     const t_uchar * path,
                      int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   struct sftp_attrs attrs;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   unsigned int pkt_len, j, count;
   char *pkt, *dir, *file;
   int len, handle_len, dots, i;
@@ -788,12 +849,12 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
   if (0 > sftp_write_pkt (pfs, soft_errors,
                           "14s", SSH_FXP_OPENDIR, ++pfs->request_id,
                           dir, str_length (dir)))
-    return 0;
+    return rel_table_nil;
   lim_free (0, dir);
 
   if (0 > (i = sftp_server_response (pfs, &pkt, &pkt_len, SSH_FXP_HANDLE,
                                      soft_errors)))
-    return 0;
+    return rel_table_nil;
 
   handle_len = READ_UINT4 (pkt + i);
   i += 4;
@@ -804,7 +865,7 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
         exit (2);
       }
       lim_free (0, pkt);
-      return 0;
+      return rel_table_nil;
     }
   handle = lim_malloc (0, handle_len);
   mem_move (handle, pkt + i, handle_len);
@@ -816,11 +877,11 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
       if (0 >sftp_write_pkt (pfs, soft_errors,
                              "14s", SSH_FXP_READDIR, ++pfs->request_id,
                              handle, handle_len))
-        return 0;
+        return rel_table_nil;
 
       if (0 > (i = sftp_server_response (pfs, &pkt, &pkt_len, SSH_FXP_NAME,
                                          soft_errors)))
-        return 0;
+        return rel_table_nil;
       if (i == 0)
         {
           lim_free (0, pkt);
@@ -843,7 +904,7 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
                   lim_free (0, pkt);
                   exit (2);
                 }
-              return 0;
+              return rel_table_nil;
             }
 
           dots = 0;
@@ -854,7 +915,7 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
           if (!dots)
             {
               file = str_save_n (0, pkt+i, len);
-              rel_add_records (&answer, rel_make_record (file, 0), 0);
+              rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str (file)), 0);
             }
 
           i += len;
@@ -870,13 +931,13 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
                   lim_free (0, pkt);
                   exit (2);
                 }
-              return 0;
+              return rel_table_nil;
             }
           i += len;
 
           if (0 > (i += sftp_import_attrs (&attrs, pkt + i, pkt_len - i,
                                            soft_errors)))
-            return 0;
+            return rel_table_nil;
         }
 
       lim_free (0, pkt);
@@ -885,12 +946,12 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
   if (0 > sftp_write_pkt (pfs, soft_errors,
                           "14s", SSH_FXP_CLOSE, ++pfs->request_id,
                           handle, handle_len))
-    return 0;
+    return rel_table_nil;
   lim_free (0, handle);
 
   if (0 > (i = sftp_server_response (pfs, &pkt, &pkt_len, SSH_FXP_HANDLE,
                                      soft_errors)))
-    return 0;
+    return rel_table_nil;
 
   if (i >= 0)
       lim_free (0, pkt);
@@ -902,7 +963,9 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
 
 /* take an sftp attr string and initialize a structure from it */
 static int
-sftp_import_attrs (struct sftp_attrs *a, char *pkt, unsigned int pkt_len,
+sftp_import_attrs (struct sftp_attrs *a,
+                   char * pkt,
+                   unsigned int pkt_len,
                    int soft_errors)
 {
   int count, i, j;
@@ -969,8 +1032,12 @@ sftp_import_attrs (struct sftp_attrs *a, char *pkt, unsigned int pkt_len,
   return i;
 }
 
-static int pfs_put_file (struct arch_pfs_session *p, t_uchar * path,
-                         mode_t perms, int data_fd, int soft_errors)
+static int
+pfs_put_file (struct arch_pfs_session *p,
+              const t_uchar * path,
+              mode_t perms,
+              int data_fd,
+              int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   struct sftp_attrs attrs;
@@ -1097,7 +1164,9 @@ static int pfs_put_file (struct arch_pfs_session *p, t_uchar * path,
 }
 
 static int
-pfs_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t mode,
+pfs_mkdir (struct arch_pfs_session * p,
+           const t_uchar * path,
+           mode_t mode,
            int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
@@ -1164,7 +1233,8 @@ pfs_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t mode,
 
 
 static int
-pfs_file_exists (struct arch_pfs_session * p, t_uchar * path)
+pfs_file_exists (struct arch_pfs_session * p,
+                 const t_uchar * path)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   struct sftp_attrs attrs;
@@ -1196,7 +1266,8 @@ pfs_file_exists (struct arch_pfs_session * p, t_uchar * path)
 
 
 static int
-pfs_is_dir (struct arch_pfs_session * p, t_uchar * path)
+pfs_is_dir (struct arch_pfs_session * p,
+            const t_uchar * path)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   int ret, i;
@@ -1236,8 +1307,11 @@ pfs_is_dir (struct arch_pfs_session * p, t_uchar * path)
 
 
 static int
-pfs_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from,
-            t_uchar * to, int soft_errors)
+pfs_rename (struct arch_pfs_session * p,
+            t_uchar ** errstr,
+            const t_uchar * from,
+            const t_uchar * to,
+            int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   int ret, i;
@@ -1294,7 +1368,9 @@ pfs_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from,
 
 
 static int
-pfs_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_rmdir (struct arch_pfs_session * p,
+           const t_uchar * path,
+           int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
 
@@ -1337,7 +1413,9 @@ pfs_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
 }
 
 static int
-pfs_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_rm (struct arch_pfs_session * p,
+        const t_uchar * path,
+        int soft_errors)
 {
   struct arch_pfs_sftp_session * pfs = (struct arch_pfs_sftp_session *)p;
   int ret, i;
@@ -1366,6 +1444,7 @@ pfs_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
   return i;
 }
 
+
 static t_uchar *
 dirfold (t_uchar *dir)
 {
@@ -1400,15 +1479,21 @@ dirfold (t_uchar *dir)
   return dir;
 }
 
-extern int arch_pfs_sftp_parse_uri (t_uchar ** user, t_uchar ** hostname, t_uchar ** port, char ** path, t_uchar * uri)
+
+int
+arch_pfs_sftp_parse_uri (t_uchar ** user,
+                         t_uchar ** hostname,
+                         t_uchar ** port,
+                         char ** path,
+                         const t_uchar * uri)
 {
-  t_uchar * start;
-  t_uchar * stop;
+  const t_uchar * start;
+  const t_uchar * stop;
   t_uchar * host_part = 0;
-  t_uchar * at;
-  t_uchar * h0;
-  t_uchar * h1;
-  t_uchar * colon;
+  const t_uchar * at;
+  const t_uchar * h0;
+  const t_uchar * h1;
+  const t_uchar * colon;
 
   /* [user@]hostname[:port]/path
    * ^    ^ ^       ^      ^
