@@ -21,7 +21,6 @@
 #include "tla/libfsutils/file-contents.h"
 #include "tla/libfsutils/tmp-files.h"
 #include "tla/libawk/associative.h"
-#include "tla/libawk/trim.h"
 #include "tla/libarch/my.h"
 #include "tla/libarch/namespace.h"
 #include "tla/libarch/archives.h"
@@ -30,9 +29,14 @@
 
 /* __STDC__ prototypes for static functions */
 static void exec_shell_command (t_uchar * command);
-static t_uchar * archive_signing_rule_file (t_uchar * archive, int no_default);
-static t_uchar * archive_signature_checking_rule_file (t_uchar * archive, int no_default);
-static int arch_pfs_memoize_checksum_file (t_uchar * archive, t_uchar * official_archive, t_uchar * revision, t_uchar * file_contents);
+static t_uchar * archive_signing_rule_file (const t_uchar * archive,
+                                            int no_default);
+static t_uchar * archive_signature_checking_rule_file (const t_uchar * archive,
+                                                       int no_default);
+static int arch_pfs_memoize_checksum_file (const t_uchar * archive,
+                                           const t_uchar * official_archive,
+                                           const t_uchar * revision,
+                                           const t_uchar * file_contents);
 
 
 
@@ -50,9 +54,13 @@ arch_pfs_has_signing_rule (t_uchar * archive)
 }
 
 
-
 int
-arch_pfs_sign_for_archive (t_uchar * archive, t_uchar * official_archive, t_uchar * revision, t_uchar * sigfile, int in_fd, int out_fd)
+arch_pfs_sign_for_archive (const t_uchar * archive,
+                           const t_uchar * official_archive,
+                           const t_uchar * revision,
+                           const t_uchar * sigfile,
+                           int in_fd,
+                           int out_fd)
 {
   t_uchar * rule_file = 0;
   t_uchar * rule = 0;
@@ -94,22 +102,21 @@ arch_pfs_sign_for_archive (t_uchar * archive, t_uchar * official_archive, t_ucha
       return answer;
     }
 
-  rule = file_contents (rule_file);
-  rule = trim_surrounding_ws (rule);
+  rule = file_contents_trimming (rule_file);
 
   if (arch_valid_archive_name (rule))
     {
       /* Copy signature from another archive. */
       struct arch_archive * sig_from = 0;
       struct arch_pfs_archive * pfs_sig_from;
-      t_uchar * revdir = 0;
+      t_uchar * revdir_from = 0;
       t_uchar * sigpath = 0;
       t_uchar * signed_checksum = 0;
 
       sig_from = arch_archive_connect (rule, 0);
 
-      revdir = arch_fs_archive_revision_path (0, revision);
-      sigpath = file_name_in_vicinity (0, revdir, sigfile);
+      revdir_from = arch_fs_archive_revision_path (sig_from, 0, revision);
+      sigpath = file_name_in_vicinity (0, revdir_from, sigfile);
 
       if (str_cmp (sig_from->vtable->type, "pfs"))
         {
@@ -163,7 +170,7 @@ arch_pfs_sign_for_archive (t_uchar * archive, t_uchar * official_archive, t_ucha
 
       arch_archive_close (sig_from);
       lim_free (0, signed_checksum);
-      lim_free (0, revdir);
+      lim_free (0, revdir_from);
       lim_free (0, sigpath);
       return 0;
     }
@@ -243,7 +250,8 @@ arch_pfs_sign_for_archive (t_uchar * archive, t_uchar * official_archive, t_ucha
 
 
 int
-arch_pfs_check_signature_for_archive (t_uchar * archive, t_uchar * signed_message)
+arch_pfs_check_signature_for_archive (const t_uchar * archive,
+                                      const t_uchar * signed_message)
 {
   t_uchar * rule_file = 0;
   t_uchar * rule = 0;
@@ -337,7 +345,6 @@ arch_pfs_check_signature_for_archive (t_uchar * archive, t_uchar * signed_messag
 }
 
 
-
 static void
 exec_shell_command (t_uchar * command)
 {
@@ -356,7 +363,8 @@ exec_shell_command (t_uchar * command)
 
 
 static t_uchar * 
-archive_signing_rule_file (t_uchar * archive, int no_default)
+archive_signing_rule_file (const t_uchar * archive,
+                           int no_default)
 {
   t_uchar * home;
   t_uchar * params_dir = 0;
@@ -397,7 +405,8 @@ archive_signing_rule_file (t_uchar * archive, int no_default)
 
 
 static t_uchar *
-archive_signature_checking_rule_file (t_uchar * archive, int no_default)
+archive_signature_checking_rule_file (const t_uchar * archive,
+                                      int no_default)
 {
   static assoc_table already_checked = 0;
   static assoc_table already_know = 0;
@@ -406,15 +415,16 @@ archive_signature_checking_rule_file (t_uchar * archive, int no_default)
   t_uchar * home;
   t_uchar * params_dir = 0;
   t_uchar * rules_dir = 0;
+  const t_uchar * maybe_answer;
   t_uchar * answer = 0;
 
-  answer = assoc_ref (already_know, archive);
+  maybe_answer = assoc_get_str_taking (already_know, rel_make_field_str (archive));
 
-  if (answer)
+  if (maybe_answer)
     {
-      answer = str_save (0, answer);
+      answer = str_save (0, maybe_answer);
     }
-  else if (!assoc_ref (already_checked, archive))
+  else if (!assoc_get_str_taking (already_checked, rel_make_field_str (archive)))
     {
       check_rule_name = str_alloc_cat (0, archive, ".check");
 
@@ -446,10 +456,10 @@ archive_signature_checking_rule_file (t_uchar * archive, int no_default)
 
       if (answer || !no_default)
         {
-          assoc_set (&already_checked, archive, "yes");
+          assoc_set_taking (&already_checked, rel_make_field_str (archive), rel_make_field_str ("yes"));
           if (answer)
             {
-              assoc_set (&already_know, archive, answer);
+              assoc_set_taking (&already_know, rel_make_field_str (archive), rel_make_field_str (answer));
             }
           else
             {
@@ -505,14 +515,15 @@ static assoc_table checksummed_revisions = 0;
 
 
 int
-arch_pfs_ensure_checksum_data (struct arch_pfs_archive * arch, t_uchar * revision)
+arch_pfs_ensure_checksum_data (struct arch_pfs_archive * arch,
+                               const t_uchar * revision)
 {
   int answer = 0;
   t_uchar * revision_key = 0;
 
   revision_key = str_alloc_cat_many (0, arch->arch.name, " ", revision, str_end);
 
-  if (!assoc_ref (checked_revisions, revision_key))
+  if (!assoc_get_str_taking (checked_revisions, rel_make_field_str (revision_key)))
     {
       t_uchar * revision_dir = 0;
 #     define N_CHECKSUM_FILES 2
@@ -547,7 +558,7 @@ arch_pfs_ensure_checksum_data (struct arch_pfs_archive * arch, t_uchar * revisio
           answer = -1;
         }
 
-      revision_dir = arch_fs_archive_revision_path (0, revision);
+      revision_dir = arch_fs_archive_revision_path (&arch->arch, 0, revision);
 
       for (x = 0; x < N_CHECKSUM_FILES; ++x)
         {
@@ -559,7 +570,7 @@ arch_pfs_ensure_checksum_data (struct arch_pfs_archive * arch, t_uchar * revisio
 
           if (file_contents)
             {
-              assoc_set (&checksummed_revisions, revision_key, "yes");
+              assoc_set_taking (&checksummed_revisions, rel_make_field_str (revision_key), rel_make_field_str ("yes"));
               found_checksums = 1;
             }
 
@@ -602,7 +613,7 @@ arch_pfs_ensure_checksum_data (struct arch_pfs_archive * arch, t_uchar * revisio
           lim_free (0, file_contents);
         }
 
-      assoc_set (&checked_revisions, revision_key, "yes");
+      assoc_set_taking (&checked_revisions, rel_make_field_str (revision_key), rel_make_field_str ("yes"));
 
       /* If the archive is supposed to be signed, but no checksums 
        * were found, fail.   If the archive is not signed, but no 
@@ -657,7 +668,10 @@ arch_pfs_ensure_checksum_data (struct arch_pfs_archive * arch, t_uchar * revisio
 
 
 static int
-arch_pfs_memoize_checksum_file (t_uchar * archive, t_uchar * official_archive, t_uchar * revision, t_uchar * file_contents)
+arch_pfs_memoize_checksum_file (const t_uchar * archive,
+                                const t_uchar * official_archive,
+                                const t_uchar * revision,
+                                const t_uchar * file_contents)
 {
   static char file_pattern[] = ("^Signature-for: (.*)/(.*)\n"
                                 "([[:(md5 [^/[:space:]]+ ................................\n):]]*)");
@@ -687,8 +701,8 @@ arch_pfs_memoize_checksum_file (t_uchar * archive, t_uchar * official_archive, t
 
 
   {
-    t_uchar * item_start;
-    t_uchar * boundary;
+    const t_uchar * item_start;
+    const t_uchar * boundary;
     t_uchar * key_prefix = 0;
 
     item_start = file_contents + pmatch[3].rm_so;
@@ -712,7 +726,7 @@ arch_pfs_memoize_checksum_file (t_uchar * archive, t_uchar * official_archive, t
 
         item_start = item_end + 34; /* space 32-byte-md5-string newline */
 
-        assoc_set (&remembered_md5s, assoc_key, md5);
+        assoc_set_taking (&remembered_md5s, rel_make_field_str (assoc_key), rel_make_field_str (md5));
 
         lim_free (0, file_name);
         lim_free (0, assoc_key);
@@ -725,15 +739,18 @@ arch_pfs_memoize_checksum_file (t_uchar * archive, t_uchar * official_archive, t
   return 0;
 }
 
+
 int
-arch_pfs_checksum_anticipates_file (t_uchar * archive, t_uchar * revision, t_uchar * file)
+arch_pfs_checksum_anticipates_file (const t_uchar * archive,
+                                    const t_uchar * revision,
+                                    const t_uchar * file)
 {
   t_uchar * key = 0;
   int answer;
 
   key = str_alloc_cat_many (0, archive, " ", revision, " ", file, str_end);
 
-  answer = !!assoc_ref (remembered_md5s, key);
+  answer = !!assoc_get_str_taking (remembered_md5s, rel_make_field_str (key));
 
   lim_free (0, key);
   return answer;
@@ -741,7 +758,8 @@ arch_pfs_checksum_anticipates_file (t_uchar * archive, t_uchar * revision, t_uch
 
 
 int
-arch_pfs_checksum_governs (t_uchar * archive, t_uchar * revision)
+arch_pfs_checksum_governs (const t_uchar * archive,
+                           const t_uchar * revision)
 {
   t_uchar * key = 0;
   int answer;
@@ -753,7 +771,7 @@ arch_pfs_checksum_governs (t_uchar * archive, t_uchar * revision)
 
   key = str_alloc_cat_many (0, archive, " ", revision, str_end);
 
-  answer = !!assoc_ref (checksummed_revisions, key);
+  answer = !!assoc_get_str_taking (checksummed_revisions, rel_make_field_str (key));
 
   lim_free (0, key);
   return answer;
@@ -781,7 +799,9 @@ arch_pfs_checksum_governs_strictly (struct arch_pfs_archive * arch)
 
 
 t_uchar *
-arch_pfs_checked_file_contents (struct arch_pfs_archive * arch, t_uchar * revision, t_uchar * path)
+arch_pfs_checked_file_contents (struct arch_pfs_archive * arch,
+                                const t_uchar * revision,
+                                const t_uchar * path)
 {
   if (!arch_pfs_checksum_governs (arch->arch.name, revision))
     return arch_pfs_file_contents (arch->pfs, path, 0);
@@ -789,13 +809,13 @@ arch_pfs_checked_file_contents (struct arch_pfs_archive * arch, t_uchar * revisi
     {
       t_uchar * tail = 0;
       t_uchar * key = 0;
-      t_uchar * remembered_md5;
+      const t_uchar * remembered_md5;
       t_uchar * contents = 0;
       t_uchar * contents_md5 = 0;
 
       tail = file_name_tail (0, path);
       key = str_alloc_cat_many (0, arch->arch.name, " ", revision, " ", tail, str_end);
-      remembered_md5 = assoc_ref (remembered_md5s, key);
+      remembered_md5 = assoc_get_str_taking (remembered_md5s, rel_make_field_str (key));
 
       contents = arch_pfs_file_contents (arch->pfs, path, 0);
       contents_md5 = md5_ascii_for_str (0, contents);
@@ -829,7 +849,10 @@ arch_pfs_checked_file_contents (struct arch_pfs_archive * arch, t_uchar * revisi
 
 
 void
-arch_pfs_checked_get_file (struct arch_pfs_archive * arch, t_uchar * revision, int out_fd, t_uchar * path)
+arch_pfs_checked_get_file (struct arch_pfs_archive * arch,
+                           const t_uchar * revision,
+                           int out_fd,
+                           const t_uchar * path)
 {
   if (!arch_pfs_checksum_governs (arch->arch.name, revision))
     arch_pfs_get_file (arch->pfs, out_fd, path, 0);
@@ -837,7 +860,7 @@ arch_pfs_checked_get_file (struct arch_pfs_archive * arch, t_uchar * revision, i
     {
       t_uchar * tail = 0;
       t_uchar * key = 0;
-      t_uchar * remembered_md5;
+      const t_uchar * remembered_md5;
       t_uchar * tmp_path = 0;
       int tmp_fd = -1;
       t_uchar buf[1 << 16];
@@ -847,7 +870,7 @@ arch_pfs_checked_get_file (struct arch_pfs_archive * arch, t_uchar * revision, i
       
       tail = file_name_tail (0, path);
       key = str_alloc_cat_many (0, arch->arch.name, " ", revision, " ", tail, str_end);
-      remembered_md5 = assoc_ref (remembered_md5s, key);
+      remembered_md5 = assoc_get_str_taking (remembered_md5s, rel_make_field_str (key));
 
       tmp_path = tmp_file_name ("/tmp", "arch-file");
       tmp_fd = safe_open (tmp_path, O_RDWR | O_CREAT | O_EXCL, 0);
