@@ -18,6 +18,7 @@
 #include "tla/libfsutils/tmp-files.h"
 #include "tla/libfsutils/rmrf.h"
 #include "tla/libfsutils/copy-file.h"
+#include "tla/libarch/project-tree.h"
 #include "tla/libarch/invent.h"
 #include "tla/libarch/build-revision.h"
 #include "tla/libarch/inode-sig.h"
@@ -33,12 +34,13 @@ arch_make_pristine (const t_uchar * tree_root,
 {
   rel_table inventory = rel_table_nil;
   t_uchar * tmp_pristine = 0;
+  const int full_meta = arch_tree_has_meta_flag (tree_root);
 
   inventory = arch_source_inventory (tree_root, 1, 0, 0);
   tmp_pristine = tmp_file_name (tree_root, ",,pristine");
   rmrf_file (tmp_pristine);
   safe_mkdir (tmp_pristine, 0777);
-  copy_file_list (tmp_pristine, tree_root, inventory);
+  copy_file_list (tmp_pristine, tree_root, inventory, full_meta);
 
   arch_install_pristine (tree_root, archive, revision, tmp_pristine);
 
@@ -214,7 +216,7 @@ arch_pristines (const t_uchar * tree_root,
   t_uchar * category = 0;
   t_uchar * branch = 0;
   t_uchar * version = 0;
-  const t_uchar * revision;
+  const t_uchar * revision = 0;
   t_uchar * locked_in_this_tree_path = 0;
   t_uchar * unlocked_in_this_tree_path = 0;
   rel_table answer = rel_table_nil;
